import React from 'react';
export interface SavedMessageProps {
    showSavedMessageRef: React.MutableRefObject<() => void>;
    className?: string;
}
/**
 * Shows a "✔ Saved" message next to your "Save changes" button to show that
 * user that the change went through. The message fades in and out.
 *
 * ```
 * const showSavedMessageRef = useRef(() => {})
 *
 * function saveChanges() {
 *     // call the API
 *
 *     showSavedMessageRef.current()
 * }
 *
 * return <div>
 *     ...
 *      <SavedMessage
 *          showSavedMessageRef={showSavedMessageRef}
 *          className="saved-message-me"
 *      />
 *     ...
 * </div>
 * ```
 */
export declare function SavedMessage({ showSavedMessageRef, className, }: SavedMessageProps): React.ReactElement;
