import React from 'react';
import { LinkProps } from 'react-router-dom';
export declare type TdLink = React.FunctionComponent<Omit<LinkProps, 'to'>>;
/**
 * Makes a table cell into a link.
 *
 * `TdLink` has styles that are required for it to work correctly.
 *
 * ```
 * <table className="table table-hover table-td-link">
 *     <tbody>
 *         {products.map(p => {
 *             const Td = getTdLink('/product/detail/' + p.id)
 *
 *             return (
 *                 <tr key={p.id}>
 *                     <Td>{p.id}</Td>
 *                     <Td>{p.name}</Td>
 *                     <Td>{p.stock}</Td>
 *                 </tr>
 *             )
 *          })}
 *     </tbody>
 * </table>
 * ```
 */
export declare function getTdLink(to: string, tdProps?: React.HTMLProps<HTMLTableCellElement>): TdLink;
