/**
 * Returns true if all values in the object are true. For use with
 * [[`useReadiness`]].
 */
export declare function allReady<T extends {
    [K in keyof T]: boolean;
}>(readiness: T): boolean;
/**
 * Used to call `onReady` when all of the components/queries in a page are ready.
 *
 * ```
 * const [onChildReady, readiness] = useReadiness(
 *     {
 *         a: false, b: false, c: false
 *     },
 *     readiness => {
 *         if (allReady(readiness)) {
 *             onReady({ ... })
 *         }
 *     },
 * )
 * ```
 */
export declare function useReadiness<TReadiness>(defaultValue: TReadiness, onChange: (readiness: TReadiness) => void): [(delta: Partial<TReadiness>) => void, TReadiness];
