import React from 'react';
import moment from 'moment-timezone';
import { Validator, UseValidationProps } from '@interface-technologies/iti-react-core';
/**
 * Get the initial value for a [[`DateInputNoPicker`]] component.
 *
 * @param timeZone an IANA time zone or `'local'`, which will use `moment.tz.guess()` as
 * the time zone.
 * @returns a formatted date string or `''` if the input moment was invalid
 */
export declare function formatDateInputNoPickerValue(m: moment.Moment, options: {
    includesTime: boolean;
    timeZone: string;
}): string;
/**
 * Parse the `value` of a [[`DateInputNoPicker`]] component.
 *
 * @param timeZone an IANA time zone or `'local'`, which will use `moment.tz.guess()` as
 * the time zone.
 * @returns a Moment object or undefined if the `value` was invalid
 */
export declare function parseDateInputNoPickerValue(value: string, options: {
    includesTime: boolean;
    timeZone: string;
}): moment.Moment | undefined;
declare function required(options: {
    includesTime: boolean;
}): Validator<string>;
/** Validators for use with `DateInputNoPicker`. */
export declare const DateInputNoPickerValidators: {
    required: typeof required;
};
export interface DateInputNoPickerProps extends UseValidationProps<string> {
    id?: string;
    placeholder?: string;
    /** This class name will be used *in addition to* form-control and the validation feedback class */
    className?: string;
    includesTime?: boolean;
    enabled?: boolean;
    readOnly?: boolean;
}
/**
 * A datetime input without a datepicker popover. Its value is just a `string`.
 * Good for things like date of birth. See also [[`DateInput`]].
 *
 * Use [[`formatDateInputNoPickerValue`]] to get the initial value for this input.
 *
 * Use [[`parseDateInputNoPickerValue`]] to get a `moment`
 * object from the string.
 */
export declare const DateInputNoPicker: React.NamedExoticComponent<DateInputNoPickerProps>;
export {};
