import React from 'react';
import { Validator, UseValidationProps } from '@interface-technologies/iti-react-core';
export interface PersonNameInputValue {
    first: string;
    middle: string;
    last: string;
}
export declare const defaultPersonNameInputValue: PersonNameInputValue;
declare function required(): Validator<PersonNameInputValue>;
/** Validators for use with [[`PersonNameInput`]]. */
export declare const PersonNameValidators: {
    required: typeof required;
};
export declare type PersonNamePersonNameInputAttributes = Omit<React.HTMLProps<HTMLInputElement>, 'disabled'>;
export declare type PersonNamePersonNameInputAttributesMap = {
    first: PersonNamePersonNameInputAttributes;
    middle: PersonNamePersonNameInputAttributes;
    last: PersonNamePersonNameInputAttributes;
};
export interface PersonNameInputProps extends UseValidationProps<PersonNameInputValue> {
    individualInputsRequired: boolean;
    fluid?: boolean;
    /** Defaults to false. */
    showMiddleNameInput?: boolean;
    /** Lets you pass attributes to each of the inputs. */
    inputAttributesMap?: Partial<PersonNamePersonNameInputAttributesMap>;
    /** Lets you control which of the inputs are enabled. */
    enabledInputs?: ('first' | 'middle' | 'last')[];
}
/**
 * A person name input that displays inputs for first name, middle name (optional), and
 * last name.
 *
 * If we just want first and last name, we will often just use two normal [[`ValidatedInputs`]]
 * instead of `PersonNameInput`.
 */
export declare const PersonNameInput: React.MemoExoticComponent<({ name, showValidation, individualInputsRequired, showMiddleNameInput, fluid, inputAttributesMap: propsInputAttributesMap, enabledInputs, ...otherProps }: PersonNameInputProps) => JSX.Element>;
export {};
