import React from 'react';
import { AddressInputFieldLengths } from './Inputs/AddressInput/AddressInputFieldLengths';
export interface ThemeColors {
    primary: string;
    secondary: string;
    success: string;
    info: string;
    warning: string;
    danger: string;
    light: string;
    dark: string;
    inputPlaceholder: string;
}
export interface ItiReactContextData {
    renderLoadingIndicator: () => React.ReactNode;
    themeColors: ThemeColors;
    fieldLengths: {
        address: AddressInputFieldLengths;
        personName: {
            prefix: number;
            first: number;
            middle: number;
            last: number;
        };
    };
    addressInput: {
        allowCanadian: boolean;
    };
    configurablePager: {
        pageSizes: number[];
    };
}
export declare type DefaultItiReactContextData = Pick<ItiReactContextData, 'themeColors' | 'fieldLengths'>;
/**
 * Defaults for **parts** of [[`ItiReactContextData`]]. Does not include defaults
 * for properties where there is no reasonable default value.
 */
export declare const defaultItiReactContextData: DefaultItiReactContextData;
/**
 * A context that provides configuration values used by code within `iti-react-core`.
 * You must wrap your application in with `<ItiReactContext.Provider>` and
 * `<ItiReactCoreContext.Provider>`.
 */
export declare const ItiReactContext: React.Context<ItiReactContextData>;
