'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('react-datepicker/dist/react-datepicker.css');
require('tippy.js/dist/tippy.css');
var itiReactCore = require('@interface-technologies/iti-react-core');
var React = require('react');
var useDebounce = require('use-debounce');
var moment = require('moment-timezone');
var DatePicker = require('react-datepicker');
var require$$1 = require('prop-types');
var lodash = require('lodash');
var Select = require('react-select');
var Color = require('color');
var AsyncSelect = require('react-select/async');
var debounce = require('debounce-promise');
var tzdb = require('@vvo/tzdb');
var useEventListener = require('@use-it/event-listener');
var bootstrap = require('bootstrap');
var reactConfirm = require('react-confirm');
var reactRouterDom = require('react-router-dom');
var freeSolidSvgIcons = require('@fortawesome/free-solid-svg-icons');
var reactFontawesome = require('@fortawesome/react-fontawesome');
var copyToClipboard = require('copy-to-clipboard');
var Tippy = require('@tippyjs/react');
var typesafeActions = require('typesafe-actions');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n["default"] = e;
    return Object.freeze(n);
}

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var moment__default = /*#__PURE__*/_interopDefaultLegacy(moment);
var DatePicker__default = /*#__PURE__*/_interopDefaultLegacy(DatePicker);
var require$$1__default = /*#__PURE__*/_interopDefaultLegacy(require$$1);
var Select__default = /*#__PURE__*/_interopDefaultLegacy(Select);
var Color__default = /*#__PURE__*/_interopDefaultLegacy(Color);
var AsyncSelect__default = /*#__PURE__*/_interopDefaultLegacy(AsyncSelect);
var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);
var useEventListener__default = /*#__PURE__*/_interopDefaultLegacy(useEventListener);
var bootstrap__namespace = /*#__PURE__*/_interopNamespace(bootstrap);
var copyToClipboard__default = /*#__PURE__*/_interopDefaultLegacy(copyToClipboard);
var Tippy__default = /*#__PURE__*/_interopDefaultLegacy(Tippy);

/**
 * Defaults for **parts** of [[`ItiReactContextData`]]. Does not include defaults
 * for properties where there is no reasonable default value.
 */
const defaultItiReactContextData = {
    themeColors: {
        primary: '#007bff',
        secondary: '#6c757d',
        success: '#28a745',
        info: '#17a2b8',
        warning: '#ffc107',
        danger: '#dc3545',
        light: '#f8f9fa',
        dark: '#343a40',
        inputPlaceholder: '#6c757d', // $gray-600,
    },
    fieldLengths: {
        address: {
            line1: 64,
            line2: 64,
            city: 64,
        },
        personName: {
            prefix: 64,
            first: 64,
            middle: 64,
            last: 64,
        },
    },
};
const throwFunction = () => {
    throw new Error('ItiReactContextData is not set.');
};
/**
 * A context that provides configuration values used by code within `iti-react-core`.
 * You must wrap your application in with `<ItiReactContext.Provider>` and
 * `<ItiReactCoreContext.Provider>`.
 */
// The default value set here should never be used
const ItiReactContext = React__default["default"].createContext({
    ...defaultItiReactContextData,
    renderLoadingIndicator: throwFunction,
    addressInput: { allowCanadian: false },
    configurablePager: { pageSizes: [10, 25, 50] },
});

/**
 * Used to show a loading indicator when async validation is progress. Uses debouncing
 * so the loading indicator is never displayed if the async validation completes quickly.
 */
function useDebouncedAsyncValidationPending(propsPending) {
    const [pending, setPending] = React.useState(false);
    const setPendingDebounced = useDebounce.useDebouncedCallback(setPending, 1000);
    React.useEffect(() => {
        if (propsPending) {
            setPendingDebounced(true);
        }
        else {
            setPendingDebounced.cancel();
            setPending(false);
        }
    }, [propsPending, setPendingDebounced]);
    return pending;
}
/**
 * Displays validation feedback below an input. Used by `ValidatedInput`, .etc.
 *
 * You usually won't use this directly unless creating your own input component.
 */
function ValidationFeedback({ validatorOutput, showValidation, children, renderLoadingIndicator, }) {
    const contextRenderLoadingIndicator = React.useContext(ItiReactContext).renderLoadingIndicator;
    renderLoadingIndicator = renderLoadingIndicator !== null && renderLoadingIndicator !== void 0 ? renderLoadingIndicator : contextRenderLoadingIndicator;
    const debouncedAsyncValidationPending = useDebouncedAsyncValidationPending(validatorOutput === itiReactCore.ASYNC_VALIDATION_PENDING);
    let feedback;
    if (showValidation) {
        if (validatorOutput === itiReactCore.ASYNC_VALIDATION_PENDING) {
            if (debouncedAsyncValidationPending) {
                feedback = (React__default["default"].createElement("div", { className: "pending-feedback" },
                    renderLoadingIndicator(),
                    " Validating..."));
            }
        }
        else if (validatorOutput && validatorOutput !== itiReactCore.INVALID_NO_FEEDBACK) {
            feedback = React__default["default"].createElement("div", { className: "invalid-feedback" }, validatorOutput);
        }
    }
    return (React__default["default"].createElement("div", { className: "validated-input" },
        children,
        feedback));
}
/**
 * Returns a Bootstrap validation class depending on `valid` and `showValidation`.
 */
function getValidationClass(valid, showValidation) {
    if (showValidation) {
        if (valid)
            return 'is-valid';
        return 'is-invalid';
    }
    return '';
}

/** @internal */
function getInvalidFeedback(includesTime) {
    let invalidFeedback = 'You must enter a valid date (MM/DD/YYYY).';
    if (includesTime) {
        invalidFeedback = 'You must enter a valid date and time.';
    }
    return invalidFeedback;
}
/**
 * The moment format string used by [[`DateInput`]] and [[`DateInputNoPicker`]] when
 * `includesTime` is false.
 */
const dateInputFormat = 'M/D/YYYY';
/** @internal */
const timeFormat = 'h:mm a';
/**
 * The moment format string used by [[`DateInput`]] and [[`DateInputNoPicker`]] when
 * `includesTime` is true.
 */
const dateTimeInputFormat = `${dateInputFormat} ${timeFormat}`;
/** @internal */
function getTimeZone(timeZone) {
    return timeZone === 'local' ? moment__default["default"].tz.guess() : timeZone;
}

// Equivalent date-fns format strings (used by react-datepicker)
const fnsDateInputFormat = 'M/d/yyyy';
const fnsTimeFormat = 'h:mm a';
const fnsDateTimeInputFormat = `${fnsDateInputFormat} ${fnsTimeFormat}`;
/**
 * @internal
 */
function convertJsDateToTimeZone(date, timeZone) {
    const str = date.toLocaleString('en-US', { timeZone });
    return new Date(str);
}
/**
 * @internal
 *
 * The offset of `date` must correspond to the browser's time zone for this to work.
 *
 * Example:
 * - assuming DST is in effect
 * - date = 9:00 am, UTC-4 = 1:00 pm UTC
 * - timeZone = America/Los_Angeles
 * - return value = 9:00 am, UTC -7 = 4:00 pm UTC
 */
function parseJsDateIgnoringTimeZone(date, timeZone) {
    const str = date.toLocaleString('en-US');
    return moment__default["default"].tz(str, 'M/D/YYYY, h:mm:ss A', timeZone);
}
//
// Validators
//
function formatValidator$1(includesTime) {
    return (value) => {
        if (value && !value.isValid())
            return getInvalidFeedback(includesTime);
        return undefined;
    };
}
function required$b(options) {
    return (value) => {
        if (!value)
            return getInvalidFeedback(options.includesTime);
        return undefined;
    };
}
/** Validators for use with [[`DateInput`]]. */
const DateValidators = {
    required: required$b,
};
/**
 * A datetime input with a datepicker popover. See also [[`DateInputNoPicker`]].
 *
 * It is implemented using `react-datepicker`.
 */
const DateInput = React__default["default"].memo(({ placeholder, includesTime = false, popperPlacement, timeIntervals, enabled = true, readOnly = false, filterDate, showValidation, name, ...otherProps }) => {
    var _a;
    const timeZone = getTimeZone(otherProps.timeZone);
    const idGuidRef = React.useRef(itiReactCore.getGuid());
    const id = (_a = otherProps.id) !== null && _a !== void 0 ? _a : idGuidRef.current;
    const { value, onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: null,
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: [formatValidator$1(includesTime), ...otherProps.validators],
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const fnsFormat = includesTime ? fnsDateTimeInputFormat : fnsDateInputFormat;
    function datePickerOnChange(date) {
        const d = date ? parseJsDateIgnoringTimeZone(date, timeZone) : null;
        onChange(d);
    }
    const classes = [
        'form-control',
        getValidationClass(!validatorOutput, showValidation),
    ];
    if (otherProps.className)
        classes.push(otherProps.className);
    const className = classes.join(' ');
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: !validatorOutput, showValidation: showValidation },
        React__default["default"].createElement(DatePicker__default["default"], { id: id, name: name, selected: value ? convertJsDateToTimeZone(value.toDate(), timeZone) : null, onChange: datePickerOnChange, className: className, dateFormat: fnsFormat, placeholderText: placeholder, popperPlacement: popperPlacement, disabledKeyboardNavigation: true, showTimeSelect: includesTime, timeIntervals: timeIntervals, timeFormat: timeFormat, filterDate: filterDate, disabled: !enabled, readOnly: readOnly, autoComplete: "off" })));
});

/**
 * Get the initial value for a [[`DateInputNoPicker`]] component.
 *
 * @param timeZone an IANA time zone or `'local'`, which will use `moment.tz.guess()` as
 * the time zone.
 * @returns a formatted date string or `''` if the input moment was invalid
 */
function formatDateInputNoPickerValue(m, options) {
    if (!(m && m.isValid()))
        return '';
    if (!options.includesTime)
        return moment__default["default"](m).format(dateInputFormat);
    return moment__default["default"](m).tz(options.timeZone).format(dateTimeInputFormat);
}
function parseValueCore(value, options) {
    // strict = true. Otherwise strings like "5" are considered valid.
    if (!options.includesTime)
        return moment__default["default"](value.trim(), dateInputFormat, true).utc().startOf('day');
    const timeZone = getTimeZone(options.timeZone);
    return moment__default["default"].tz(value.trim(), dateTimeInputFormat, true, timeZone);
}
/**
 * Parse the `value` of a [[`DateInputNoPicker`]] component.
 *
 * @param timeZone an IANA time zone or `'local'`, which will use `moment.tz.guess()` as
 * the time zone.
 * @returns a Moment object or undefined if the `value` was invalid
 */
function parseDateInputNoPickerValue(value, options) {
    if (typeof value !== 'string')
        return undefined;
    const d = parseValueCore(value, options);
    return d.isValid() ? d : undefined;
}
//
// Validators
//
function isValid(value, includesTime) {
    return parseValueCore(value, { includesTime, timeZone: 'Etc/UTC' }).isValid();
}
function formatValidator(includesTime) {
    return (value) => {
        if (value && !isValid(value, includesTime))
            return getInvalidFeedback(includesTime);
        return undefined;
    };
}
function required$a(options) {
    return (value) => {
        if (!value)
            return getInvalidFeedback(options.includesTime);
        return undefined;
    };
}
/** Validators for use with `DateInputNoPicker`. */
const DateInputNoPickerValidators = {
    required: required$a,
};
/**
 * A datetime input without a datepicker popover. Its value is just a `string`.
 * Good for things like date of birth. See also [[`DateInput`]].
 *
 * Use [[`formatDateInputNoPickerValue`]] to get the initial value for this input.
 *
 * Use [[`parseDateInputNoPickerValue`]] to get a `moment`
 * object from the string.
 */
const DateInputNoPicker = React__default["default"].memo(({ placeholder, includesTime = false, enabled = true, readOnly = false, showValidation, name, ...otherProps }) => {
    var _a;
    const idGuidRef = React.useRef(itiReactCore.getGuid());
    const id = (_a = otherProps.id) !== null && _a !== void 0 ? _a : idGuidRef.current;
    const { value, onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: '',
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: [formatValidator(includesTime), ...otherProps.validators],
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const classes = [
        'form-control',
        getValidationClass(!validatorOutput, showValidation),
    ];
    if (otherProps.className)
        classes.push(otherProps.className);
    const className = classes.join(' ');
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement("input", { id: id, name: name, value: value, onChange: (e) => onChange(e.target.value), className: className, placeholder: placeholder, disabled: !enabled, readOnly: readOnly, autoComplete: "off" })));
});

var index_commonjs$1 = {exports: {}};

var templateParser = {};

var helpers = {};

Object.defineProperty(helpers, "__esModule", {
  value: true
});
helpers.count_occurences = count_occurences;

// Counts all occurences of a symbol in a string
function count_occurences(symbol, string) {
  var count = 0; // Using `.split('')` here instead of normal `for ... of`
  // because the importing application doesn't neccessarily include an ES6 polyfill.
  // The `.split('')` approach discards "exotic" UTF-8 characters
  // (the ones consisting of four bytes)
  // but template placeholder characters don't fall into that range
  // so skipping such miscellaneous "exotic" characters
  // won't matter here for just counting placeholder character occurrences.

  for (var _iterator = string.split(''), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var character = _ref;

    if (character === symbol) {
      count++;
    }
  }

  return count;
}

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _helpers = helpers;

function _default(template, placeholder, parse) {
  if (typeof placeholder === 'function') {
    parse = placeholder;
    placeholder = 'x';
  }

  var max_characters = (0, _helpers.count_occurences)(placeholder, template);
  return function (character, value) {
    if (value.length < max_characters) {
      return parse(character, value);
    }
  };
}

}(templateParser));

var templateFormatter = {};

var closeBraces = {};

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = closeBraces;

var _helpers = helpers;

function closeBraces(retained_template, template) {
  var placeholder = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'x';
  var empty_placeholder = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : ' ';
  var cut_before = retained_template.length;
  var opening_braces = (0, _helpers.count_occurences)('(', retained_template);
  var closing_braces = (0, _helpers.count_occurences)(')', retained_template);
  var dangling_braces = opening_braces - closing_braces;

  while (dangling_braces > 0 && cut_before < template.length) {
    retained_template += template[cut_before].replace(placeholder, empty_placeholder);

    if (template[cut_before] === ')') {
      dangling_braces--;
    }

    cut_before++;
  }

  return retained_template;
}

}(closeBraces));

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _helpers = helpers;

var _closeBraces = _interopRequireDefault(closeBraces);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Takes a `template` where character placeholders
// are denoted by 'x'es (e.g. 'x (xxx) xxx-xx-xx').
//
// Returns a function which takes `value` characters
// and returns the `template` filled with those characters.
// If the `template` can only be partially filled
// then it is cut off.
//
// If `should_close_braces` is `true`,
// then it will also make sure all dangling braces are closed,
// e.g. "8 (8" -> "8 (8  )" (iPhone style phone number input).
//
function _default(template) {
  var placeholder = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'x';
  var should_close_braces = arguments.length > 2 ? arguments[2] : undefined;

  if (!template) {
    return function (value) {
      return {
        text: value
      };
    };
  }

  var characters_in_template = (0, _helpers.count_occurences)(placeholder, template);
  return function (value) {
    if (!value) {
      return {
        text: '',
        template: template
      };
    }

    var value_character_index = 0;
    var filled_in_template = ''; // Using `.split('')` here instead of normal `for ... of`
    // because the importing application doesn't neccessarily include an ES6 polyfill.
    // The `.split('')` approach discards "exotic" UTF-8 characters
    // (the ones consisting of four bytes)
    // but template placeholder characters don't fall into that range
    // and appending UTF-8 characters to a string in parts still works.

    for (var _iterator = template.split(''), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var character = _ref;

      if (character !== placeholder) {
        filled_in_template += character;
        continue;
      }

      filled_in_template += value[value_character_index];
      value_character_index++; // If the last available value character has been filled in,
      // then return the filled in template
      // (either trim the right part or retain it,
      //  if no more character placeholders in there)

      if (value_character_index === value.length) {
        // If there are more character placeholders
        // in the right part of the template
        // then simply trim it.
        if (value.length < characters_in_template) {
          break;
        }
      }
    }

    if (should_close_braces) {
      filled_in_template = (0, _closeBraces["default"])(filled_in_template, template);
    }

    return {
      text: filled_in_template,
      template: template
    };
  };
}

}(templateFormatter));

var parseDigit = {};

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;
exports.DIGITS = void 0;
// Copied from `libphonenumber-js`:
// https://gitlab.com/catamphetamine/libphonenumber-js/blob/master/source/parse.js
//
// These mappings map a character (key) to a specific digit that should
// replace it for normalization purposes. Non-European digits that
// may be used in phone numbers are mapped to a European equivalent.
//
// E.g. in Iraq they don't write `+442323234` but rather `+٤٤٢٣٢٣٢٣٤`.
//
var DIGITS = {
  '0': '0',
  '1': '1',
  '2': '2',
  '3': '3',
  '4': '4',
  '5': '5',
  '6': '6',
  '7': '7',
  '8': '8',
  '9': '9',
  "\uFF10": '0',
  // Fullwidth digit 0
  "\uFF11": '1',
  // Fullwidth digit 1
  "\uFF12": '2',
  // Fullwidth digit 2
  "\uFF13": '3',
  // Fullwidth digit 3
  "\uFF14": '4',
  // Fullwidth digit 4
  "\uFF15": '5',
  // Fullwidth digit 5
  "\uFF16": '6',
  // Fullwidth digit 6
  "\uFF17": '7',
  // Fullwidth digit 7
  "\uFF18": '8',
  // Fullwidth digit 8
  "\uFF19": '9',
  // Fullwidth digit 9
  "\u0660": '0',
  // Arabic-indic digit 0
  "\u0661": '1',
  // Arabic-indic digit 1
  "\u0662": '2',
  // Arabic-indic digit 2
  "\u0663": '3',
  // Arabic-indic digit 3
  "\u0664": '4',
  // Arabic-indic digit 4
  "\u0665": '5',
  // Arabic-indic digit 5
  "\u0666": '6',
  // Arabic-indic digit 6
  "\u0667": '7',
  // Arabic-indic digit 7
  "\u0668": '8',
  // Arabic-indic digit 8
  "\u0669": '9',
  // Arabic-indic digit 9
  "\u06F0": '0',
  // Eastern-Arabic digit 0
  "\u06F1": '1',
  // Eastern-Arabic digit 1
  "\u06F2": '2',
  // Eastern-Arabic digit 2
  "\u06F3": '3',
  // Eastern-Arabic digit 3
  "\u06F4": '4',
  // Eastern-Arabic digit 4
  "\u06F5": '5',
  // Eastern-Arabic digit 5
  "\u06F6": '6',
  // Eastern-Arabic digit 6
  "\u06F7": '7',
  // Eastern-Arabic digit 7
  "\u06F8": '8',
  // Eastern-Arabic digit 8
  "\u06F9": '9' // Eastern-Arabic digit 9

};
exports.DIGITS = DIGITS;

function _default(character, value) {
  return DIGITS[character];
}

}(parseDigit));

var parse = {};

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = parse;

// Parses the `text`.
//
// Returns `{ value, caret }` where `caret` is
// the caret position inside `value`
// corresponding to the `caret_position` inside `text`.
//
// The `text` is parsed by feeding each character sequentially to
// `parse_character(character, value)` function
// and appending the result (if it's not `undefined`) to `value`.
//
// Example:
//
// `text` is `8 (800) 555-35-35`,
// `caret_position` is `4` (before the first `0`).
// `parse_character` is `(character, value) =>
//   if (character >= '0' && character <= '9') { return character }`.
//
// then `parse()` outputs `{ value: '88005553535', caret: 2 }`.
//
function parse(text, caret_position, parse_character) {
  var value = '';
  var focused_input_character_index = 0;
  var index = 0;

  while (index < text.length) {
    var character = parse_character(text[index], value);

    if (character !== undefined) {
      value += character;

      if (caret_position !== undefined) {
        if (caret_position === index) {
          focused_input_character_index = value.length - 1;
        } else if (caret_position > index) {
          focused_input_character_index = value.length;
        }
      }
    }

    index++;
  } // If caret position wasn't specified


  if (caret_position === undefined) {
    // Then set caret position to "after the last input character"
    focused_input_character_index = value.length;
  }

  var result = {
    value: value,
    caret: focused_input_character_index
  };
  return result;
}

}(parse));

var format = {};

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = format;

var _templateFormatter = _interopRequireDefault(templateFormatter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Formats `value` value preserving `caret` at the same character.
//
// `{ value, caret }` attribute is the result of `parse()` function call.
//
// Returns `{ text, caret }` where the new `caret` is the caret position
// inside `text` text corresponding to the original `caret` position inside `value`.
//
// `formatter(value)` is a function returning `{ text, template }`.
//
// `text` is the `value` value formatted using `template`.
// It may either cut off the non-filled right part of the `template`
// or it may fill the non-filled character placeholders
// in the right part of the `template` with `spacer`
// which is a space (' ') character by default.
//
// `template` is the template used to format the `value`.
// It can be either a full-length template or a partial template.
//
// `formatter` can also be a string — a `template`
// where character placeholders are denoted by 'x'es.
// In this case `formatter` function is automatically created.
//
// Example:
//
// `value` is '880',
// `caret` is `2` (before the first `0`)
//
// `formatter` is `'880' =>
//   { text: '8 (80 )', template: 'x (xxx) xxx-xx-xx' }`
//
// The result is `{ text: '8 (80 )', caret: 4 }`.
//
function format(value, caret, formatter) {
  if (typeof formatter === 'string') {
    formatter = (0, _templateFormatter["default"])(formatter);
  }

  var _ref = formatter(value) || {},
      text = _ref.text,
      template = _ref.template;

  if (text === undefined) {
    text = value;
  }

  if (template) {
    if (caret === undefined) {
      caret = text.length;
    } else {
      var index = 0;
      var found = false;
      var possibly_last_input_character_index = -1;

      while (index < text.length && index < template.length) {
        // Character placeholder found
        if (text[index] !== template[index]) {
          if (caret === 0) {
            found = true;
            caret = index;
            break;
          }

          possibly_last_input_character_index = index;
          caret--;
        }

        index++;
      } // If the caret was positioned after last input character,
      // then the text caret index is just after the last input character.


      if (!found) {
        caret = possibly_last_input_character_index + 1;
      }
    }
  }

  return {
    text: text,
    caret: caret
  };
}

}(format));

var inputControl = {};

var edit = {};

(function (exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = edit;

// Edits text `value` (if `operation` is passed) and repositions the `caret` if needed.
//
// Example:
//
// value - '88005553535'
// caret - 2 // starting from 0; is positioned before the first zero
// operation - 'Backspace'
//
// Returns
// {
// 	value: '8005553535'
// 	caret: 1
// }
//
// Currently supports just 'Delete' and 'Backspace' operations
//
function edit(value, caret, operation) {
  switch (operation) {
    case 'Backspace':
      // If there exists the previous character,
      // then erase it and reposition the caret.
      if (caret > 0) {
        // Remove the previous character
        value = value.slice(0, caret - 1) + value.slice(caret); // Position the caret where the previous (erased) character was

        caret--;
      }

      break;

    case 'Delete':
      // Remove current digit (if any)
      value = value.slice(0, caret) + value.slice(caret + 1);
      break;
  }

  return {
    value: value,
    caret: caret
  };
}

}(edit));

var dom = {};

Object.defineProperty(dom, "__esModule", {
  value: true
});
dom.getSelection = getSelection;
dom.getOperation = getOperation;
dom.getCaretPosition = getCaretPosition;
dom.setCaretPosition = setCaretPosition;
dom.Keys = void 0;

// Gets <input/> selection bounds
function getSelection(element) {
  // If no selection, return nothing
  if (element.selectionStart === element.selectionEnd) {
    return;
  }

  return {
    start: element.selectionStart,
    end: element.selectionEnd
  };
} // Key codes


var Keys = {
  Backspace: 8,
  Delete: 46
}; // Finds out the operation to be intercepted and performed
// based on the key down event `keyCode`.

dom.Keys = Keys;

function getOperation(event) {
  switch (event.keyCode) {
    case Keys.Backspace:
      return 'Backspace';

    case Keys.Delete:
      return 'Delete';
  }
} // Gets <input/> caret position


function getCaretPosition(element) {
  return element.selectionStart;
} // Sets <input/> caret position


function setCaretPosition(element, caret_position) {
  // Sanity check
  if (caret_position === undefined) {
    return;
  } // Set caret position.
  // There has been an issue with caret positioning on Android devices.
  // https://github.com/catamphetamine/input-format/issues/2
  // I was revisiting this issue and looked for similar issues in other libraries.
  // For example, there's [`text-mask`](https://github.com/text-mask/text-mask) library.
  // They've had exactly the same issue when the caret seemingly refused to be repositioned programmatically.
  // The symptoms were the same: whenever the caret passed through a non-digit character of a mask (a whitespace, a bracket, a dash, etc), it looked as if it placed itself one character before its correct position.
  // https://github.com/text-mask/text-mask/issues/300
  // They seem to have found a basic fix for it: calling `input.setSelectionRange()` in a timeout rather than instantly for Android devices.
  // https://github.com/text-mask/text-mask/pull/400/files
  // I've implemented the same workaround here.


  if (isAndroid()) {
    setTimeout(function () {
      return element.setSelectionRange(caret_position, caret_position);
    }, 0);
  } else {
    element.setSelectionRange(caret_position, caret_position);
  }
}

function isAndroid() {
  // `navigator` is not defined when running mocha tests.
  if (typeof navigator !== 'undefined') {
    return ANDROID_USER_AGENT_REG_EXP.test(navigator.userAgent);
  }
}

var ANDROID_USER_AGENT_REG_EXP = /Android/i;

Object.defineProperty(inputControl, "__esModule", {
  value: true
});
inputControl.onCut = onCut;
inputControl.onPaste = onPaste;
inputControl.onChange = onChange;
inputControl.onKeyDown = onKeyDown;

var _edit = _interopRequireDefault(edit);

var _parse3 = _interopRequireDefault(parse);

var _format2 = _interopRequireDefault(format);

var _dom = dom;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// Deprecated.
// I don't know why this function exists.
function onCut(event, input, _parse, _format, on_change) {
  // The actual cut hasn't happened just yet hence the timeout.
  setTimeout(function () {
    return formatInputText(input, _parse, _format, undefined, on_change);
  }, 0);
} // Deprecated.
// I don't know why this function exists.


function onPaste(event, input, _parse, _format, on_change) {
  var selection = (0, _dom.getSelection)(input); // If selection is made,
  // just erase the selected text
  // prior to pasting

  if (selection) {
    eraseSelection(input, selection);
  }

  formatInputText(input, _parse, _format, undefined, on_change);
}

function onChange(event, input, _parse, _format, on_change) {
  formatInputText(input, _parse, _format, undefined, on_change);
} // "Delete" and "Backspace" keys are special
// in a way that they're not handled by the regular `onChange()` handler
// and instead are intercepted and re-applied manually.
// The reason is that normally hitting "Backspace" or "Delete"
// results in erasing a character, but that character might be any character,
// while it would be a better "user experience" if it erased not just any character
// but the closest "meaningful" character.
// For example, if a template is `(xxx) xxx-xxxx`,
// and the `<input/>` value is `(111) 222-3333`,
// then, if a user begins erasing the `3333` part via "Backspace"
// and reaches the "-" character, then it would just erase the "-" character.
// Nothing wrong with that, but it would be a better "user experience"
// if hitting "Backspace" at that position would erase the closest "meaningful"
// character, which would be the rightmost `2`.
// So, what this `onKeyDown()` handler does is it intercepts
// "Backspace" and "Delete" keys and re-applies those operations manually
// following the logic described above.


function onKeyDown(event, input, _parse, _format, on_change) {
  var operation = (0, _dom.getOperation)(event);

  switch (operation) {
    case 'Delete':
    case 'Backspace':
      // Intercept this operation and perform it manually.
      event.preventDefault();
      var selection = (0, _dom.getSelection)(input); // If a selection is made, just erase the selected text.

      if (selection) {
        eraseSelection(input, selection);
        return formatInputText(input, _parse, _format, undefined, on_change);
      } // Else, perform the (character erasing) operation manually.


      return formatInputText(input, _parse, _format, operation, on_change);

  }
}
/**
 * Erases the selected text inside an `<input/>`.
 * @param  {DOMElement} input
 * @param  {Selection} selection
 */


function eraseSelection(input, selection) {
  var text = input.value;
  text = text.slice(0, selection.start) + text.slice(selection.end);
  input.value = text;
  (0, _dom.setCaretPosition)(input, selection.start);
}
/**
 * Parses and re-formats `<input/>` textual value.
 * E.g. when a user enters something into the `<input/>`
 * that raw input must first be parsed and the re-formatted properly.
 * Is called either after some user input (e.g. entered a character, pasted something)
 * or after the user performed an `operation` (e.g. "Backspace", "Delete").
 * @param  {DOMElement} input
 * @param  {Function} parse
 * @param  {Function} format
 * @param  {string} [operation] - The operation that triggered `<input/>` textual value change. E.g. "Backspace", "Delete".
 * @param  {Function} onChange
 */


function formatInputText(input, _parse, _format, operation, on_change) {
  // Parse `<input/>` textual value.
  // Get the `value` and `caret` position.
  var _parse2 = (0, _parse3["default"])(input.value, (0, _dom.getCaretPosition)(input), _parse),
      value = _parse2.value,
      caret = _parse2.caret; // If a user performed an operation ("Backspace", "Delete")
  // then apply that operation and get the new `value` and `caret` position.


  if (operation) {
    var newValueAndCaret = (0, _edit["default"])(value, caret, operation);
    value = newValueAndCaret.value;
    caret = newValueAndCaret.caret;
  } // Format the `value`.
  // (and reposition the caret accordingly)


  var formatted = (0, _format2["default"])(value, caret, _format);
  var text = formatted.text;
  caret = formatted.caret; // Set `<input/>` textual value manually
  // to prevent React from resetting the caret position
  // later inside a subsequent `render()`.
  // Doesn't work for custom `inputComponent`s for some reason.

  input.value = text; // Position the caret properly.

  (0, _dom.setCaretPosition)(input, caret); // If the `<input/>` textual value did change,
  // then the parsed `value` may have changed too.

  on_change(value);
}

(function (module, exports) {

exports = module.exports = {};

exports.templateParser    = templateParser.default;
exports.templateFormatter = templateFormatter.default;
exports.parseDigit        = parseDigit.default;
exports.parse             = parse.default;
exports.format            = format.default;

// Deprecated.
// I don't know why these functions exist.
exports.onChange       = inputControl.onChange;
exports.onPaste        = inputControl.onPaste;

exports.onCut          = inputControl.onCut;
exports.onKeyDown      = inputControl.onKeyDown;

// exports['default'] = ...
}(index_commonjs$1, index_commonjs$1.exports));

var index_commonjs = {exports: {}};

var Input = {};

(function (exports) {

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _react = _interopRequireWildcard(React__default["default"]);

var _propTypes = _interopRequireDefault(require$$1__default["default"]);

var _inputControl = inputControl;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

// Usage:
//
// <ReactInput
// 	value={this.state.phone}
// 	onChange={phone => this.setState({ phone })}
// 	parse={character => character}
// 	format={value => ({ text: value, template: 'xxxxxxxx' })}/>
//
function Input(_ref, ref) {
  var value = _ref.value,
      parse = _ref.parse,
      format = _ref.format,
      InputComponent = _ref.inputComponent,
      onChange = _ref.onChange,
      onKeyDown = _ref.onKeyDown,
      rest = _objectWithoutProperties(_ref, ["value", "parse", "format", "inputComponent", "onChange", "onKeyDown"]);

  var internalRef = (0, _react.useRef)();
  var setRef = (0, _react.useCallback)(function (instance) {
    internalRef.current = instance;

    if (ref) {
      if (typeof ref === 'function') {
        ref(instance);
      } else {
        ref.current = instance;
      }
    }
  }, [ref]);

  var _onChange = (0, _react.useCallback)(function (event) {
    return (0, _inputControl.onChange)(event, internalRef.current, parse, format, onChange);
  }, [internalRef, parse, format, onChange]);

  var _onKeyDown = (0, _react.useCallback)(function (event) {
    if (onKeyDown) {
      onKeyDown(event);
    }

    return (0, _inputControl.onKeyDown)(event, internalRef.current, parse, format, onChange);
  }, [internalRef, parse, format, onChange, onKeyDown]);

  return _react["default"].createElement(InputComponent, _extends({}, rest, {
    ref: setRef,
    value: format(isEmptyValue(value) ? '' : value).text,
    onKeyDown: _onKeyDown,
    onChange: _onChange
  }));
}

Input = _react["default"].forwardRef(Input);
Input.propTypes = {
  // Parses a single characher of `<input/>` text.
  parse: _propTypes["default"].func.isRequired,
  // Formats `value` into `<input/>` text.
  format: _propTypes["default"].func.isRequired,
  // Renders `<input/>` by default.
  inputComponent: _propTypes["default"].elementType.isRequired,
  // `<input/>` `type` attribute.
  type: _propTypes["default"].string.isRequired,
  // Is parsed from <input/> text.
  value: _propTypes["default"].string,
  // This handler is called each time `<input/>` text is changed.
  onChange: _propTypes["default"].func.isRequired,
  // Passthrough
  onKeyDown: _propTypes["default"].func,
  onCut: _propTypes["default"].func,
  onPaste: _propTypes["default"].func
};
Input.defaultProps = {
  // Renders `<input/>` by default.
  inputComponent: 'input',
  // `<input/>` `type` attribute.
  type: 'text'
};
var _default = Input;
exports["default"] = _default;

function isEmptyValue(value) {
  return value === undefined || value === null;
}

}(Input));

(function (module, exports) {

exports = module.exports = Input.default;

exports['default'] = Input.default;
}(index_commonjs, index_commonjs.exports));

var ReactInput = index_commonjs.exports;

const { template, lenWithCountryCode, visibleLen, formatter } = itiReactCore.AdvancedPhoneNumberUtil;
// eslint-disable-next-line
const parser = index_commonjs$1.exports.templateParser(template, index_commonjs$1.exports.parseDigit);
/** Validates a phone number. Automatically used by [[`PhoneInput`]]. */
const phoneInputValidator = (value) => {
    if (value && itiReactCore.normalizePhoneNumber(value).length !== lenWithCountryCode)
        return `The phone number must have exactly ${visibleLen} digits.`;
    return undefined;
};
/**
 * A validated phone number input. It automatically adds the US/Canada country code (1)
 * to the value. Other country codes are not currently supported.
 *
 * You don't have to pass in a validator that checks that the phone number is valid —
 * this is done automatically. `PhoneInput` can be used with `Validators.required()`.
 */
function PhoneInput({ id, showValidation, enabled = true, name, inputAttributes = {}, ...props }) {
    const { value, onChange: _onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: '',
    });
    function onChange(newValue) {
        _onChange(newValue ? itiReactCore.normalizePhoneNumber(newValue) : '');
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators: [phoneInputValidator, ...props.validators],
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    const normalized = itiReactCore.normalizePhoneNumber(value);
    let noCountryCode = normalized;
    if (normalized.length > 0) {
        noCountryCode = normalized.substring(1);
    }
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement("input", { name: name, value: normalized, type: "hidden" }),
        React__default["default"].createElement(ReactInput, { id: id, name: `${name}__display`, disabled: !enabled, onChange: onChange, value: noCountryCode, 
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            parse: parser, format: formatter, className: `form-control ${getValidationClass(!validatorOutput, showValidation)}`, ...inputAttributes })));
}

/**
 * Takes in a list of normal [[`SelectOption`]]s and "group" select options
 * and flattens it out into a list of normal [[`SelectOption`]]s.
 */
function getNonGroupOptions(options) {
    const [groupOptions, nonGroupOptions] = lodash.partition(options, (o) => typeof o.value === 'undefined');
    return [
        ...nonGroupOptions,
        ...lodash.flatten(groupOptions.map((go) => go.options)),
    ];
}
/**
 * @internal
 *
 * Makes it so typing in one of our select components filters the options by the `label`
 * only, not the `label` and the `value` which is the default behavior.
 *
 * See https://react-select.com/advanced#custom-filter-logic.
 */
const filterOption = Select.createFilter({
    stringify: (o) => o.label,
});

/* eslint-disable @typescript-eslint/no-unsafe-member-access */
/** Styles the `react-select` component to match Bootstrap form-control inputs. */
const getSelectStyles = (options) => {
    const { valid, showValidation, themeColors, width, formControlSize, isMulti } = options;
    const disabledDarkenBy = 0.15;
    const indicatorPaddingSmY = '4px';
    const indicatorPaddingSmAll = `${indicatorPaddingSmY} 6px`;
    const multiValueMarginLgX = '4px';
    const noStyles = (base) => base;
    function getSvgStyles(defaultDim) {
        if (formControlSize === 'lg') {
            // Scale up SVG icons to match the larger control
            const scaleFactor = 1.25; // = $font-size-lg / $font-size-base
            const scaledDim = scaleFactor * defaultDim;
            return { width: scaledDim, height: scaledDim };
        }
        return {};
    }
    return {
        control: (base, props) => {
            const styles = { ...base };
            if (typeof width === 'number')
                styles.width = width;
            styles.borderColor = '#ced4da'; // $gray-400
            if (props.isDisabled) {
                styles.backgroundColor = '#e9ecef'; // $gray-200
            }
            else {
                styles.backgroundColor = 'white';
            }
            const primaryColor = new Color__default["default"](themeColors.primary);
            const dangerColor = new Color__default["default"](themeColors.danger);
            const successColor = new Color__default["default"](themeColors.success);
            if (showValidation) {
                const borderColor = valid ? successColor : dangerColor;
                styles.borderColor = borderColor.toString();
                styles['&:hover'].borderColor = borderColor.toString();
            }
            if (props.isFocused) {
                const borderColor = primaryColor.lighten(0.25);
                const boxShadowColor = primaryColor.alpha(0.25);
                styles.borderColor = borderColor.toString();
                styles['&:hover'].borderColor = borderColor.toString();
                styles.boxShadow = `0 0 0 0.2rem ${boxShadowColor.toString()}`;
            }
            if (formControlSize) {
                delete styles.minHeight;
                if (formControlSize === 'sm') {
                    styles.height = 'calc(1.8125rem + 2px)';
                    styles.fontSize = '0.875rem';
                    styles.lineHeight = '1.5';
                    styles.borderRadius = '0.2rem';
                }
                else if (formControlSize === 'lg') {
                    styles.height = 'calc(2.875rem + 2px)';
                    styles.fontSize = '1.25rem';
                    styles.lineHeight = '1.5';
                    styles.borderRadius = '0.3rem';
                }
            }
            return styles;
        },
        placeholder: (base, props) => {
            if (props.isDisabled)
                return base;
            return {
                ...base,
                color: themeColors.inputPlaceholder,
            };
        },
        dropdownIndicator: (base, props) => {
            const styles = { ...base };
            if (props.isDisabled) {
                styles.color = new Color__default["default"](base.color).darken(disabledDarkenBy).toString();
            }
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        clearIndicator: (base) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        loadingIndicator: (base) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(20);
            if (formControlSize === 'sm')
                styles.padding = indicatorPaddingSmAll;
            return styles;
        },
        indicatorSeparator: (base, props) => {
            const styles = { ...base };
            if (!(props.hasValue && props.selectProps.isClearable)) {
                styles.display = 'none';
            }
            if (props.isDisabled) {
                styles.backgroundColor = new Color__default["default"](base.backgroundColor)
                    .darken(disabledDarkenBy)
                    .toString();
            }
            if (formControlSize === 'sm') {
                styles.marginTop = indicatorPaddingSmY;
                styles.marginBottom = indicatorPaddingSmY;
            }
            return styles;
        },
        menu: (base) => ({
            ...base,
            zIndex: 1000, // Value of $zindex-dropdown in the Bootstrap z-index master list
        }),
        valueContainer: (base, props) => {
            const styles = { ...base };
            if (formControlSize === 'sm') {
                styles.height = '1.8125rem';
                let paddingY = '0.25rem';
                if (props.isMulti)
                    paddingY = '0';
                // -2px because placeholder/option has 2px horiziontal margin
                styles.padding = `${paddingY} calc(0.5rem - 2px)`;
            }
            else if (formControlSize === 'lg') {
                styles.height = '2.875rem';
                let paddingY = '0.5rem';
                let optionMarginX = '2px';
                if (props.isMulti) {
                    paddingY = '0';
                    optionMarginX = multiValueMarginLgX;
                }
                // -2px because placeholder/option has 2px horiziontal margin
                styles.padding = `${paddingY} calc(1rem - ${optionMarginX})`;
            }
            return styles;
        },
        input: (base) => {
            const styles = { ...base };
            // Remove input's vertical margin and padding since we have already added padding to the valueContainer
            if (!isMulti && (formControlSize === 'sm' || formControlSize === 'lg')) {
                styles.marginTop = '0';
                styles.marginBottom = '0';
                styles.paddingTop = '0';
                styles.paddingBottom = '0';
            }
            return styles;
        },
        multiValue: (base, props) => {
            const styles = { ...base };
            if (formControlSize === 'lg') {
                styles.marginLeft = multiValueMarginLgX;
                styles.marginRight = multiValueMarginLgX;
            }
            if (props.data.isFixed)
                styles.backgroundColor = 'gray';
            return styles;
        },
        multiValueLabel: (base, props) => {
            const styles = { ...base };
            if (props.data.isFixed) {
                styles.color = 'white';
                styles.paddingRight = 6;
            }
            return styles;
        },
        multiValueRemove: (base, props) => {
            const styles = { ...base };
            styles.svg = getSvgStyles(14);
            if (props.data.isFixed)
                styles.display = 'none';
            return styles;
        },
        singleValue: (base, props) => {
            const styles = { ...base };
            if (props.isDisabled) {
                styles.color = '#495057'; // $input-color == $gray-700
            }
            return styles;
        },
        /* Return a function for EVERY part of the select that can be styled. This way,
         * applications that use iti-react can write their own styling functions
         * that build on top of this one, like:
         *
         * return {
         *      control: (base, state) => {
         *          let styles = getSelectStyles(...).control(base, state)
         *
         *          // do stuff to styles
         *
         *          return styles
         *      }
         * }
         *
         * Then, in the future, we can add new styles here and any functions built on top of this one
         * will automatically utilize the new styles.
         */
        container: noStyles,
        group: noStyles,
        groupHeading: noStyles,
        indicatorsContainer: noStyles,
        loadingMessage: noStyles,
        menuList: noStyles,
        noOptionsMessage: noStyles,
        option: noStyles,
        menuPortal: noStyles,
    };
};

/** @internal */
const defaultSelectProps = {
    enabled: true,
    isClearable: false,
    getStyles: getSelectStyles,
    menuPlacement: 'auto',
};

/**
 * A validated dropdown component based on `react-select`.
 *
 * This component is expensive to render so use `React.memo` when necessary.
 */
const ValidatedSelect = React__default["default"].memo(({ id, name, options, validators, showValidation, placeholder, className, formControlSize, width, components, isLoading, enabled = defaultSelectProps.enabled, isClearable = defaultSelectProps.isClearable, getStyles = defaultSelectProps.getStyles, isOptionEnabled, menuIsOpen, onMenuOpen, onMenuClose, menuPlacement = defaultSelectProps.menuPlacement, ...props }) => {
    const { value, onChange: _onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: null,
    });
    function onChange(option0, actionMeta) {
        // option will be an array if the user presses backspace
        // This is so that if isClearable = false, null will never be passed to the
        // onChange prop
        if (!isClearable && actionMeta.action === 'pop-value')
            return;
        const option = option0;
        // Be careful with the conditional - option.value could be 0
        let newValue = null;
        if (option && option.value !== null) {
            newValue = option.value;
        }
        _onChange(newValue);
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators,
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    const { themeColors } = React.useContext(ItiReactContext);
    const nonGroupOptions = getNonGroupOptions(options);
    let selectValue = null;
    // Be careful: value can be 0
    if (value !== null) {
        const findResult = nonGroupOptions.find((o) => o.value === value);
        if (findResult)
            selectValue = findResult;
    }
    let isOptionDisabled;
    if (isOptionEnabled)
        isOptionDisabled = (o) => !isOptionEnabled(o);
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement(Select__default["default"], { name: name, className: className, inputId: id, options: options, placeholder: placeholder, value: selectValue, onChange: onChange, isClearable: isClearable, isDisabled: !enabled, isLoading: isLoading, isOptionDisabled: isOptionDisabled, styles: getStyles({
                valid: !validatorOutput,
                showValidation,
                themeColors,
                width,
                formControlSize,
                isMulti: false,
            }), "aria-label": props['aria-label'], "aria-labelledby": props['aria-labelledby'], 
            // eslint-disable-next-line -- TODO:SAM
            components: components, menuIsOpen: menuIsOpen, onMenuOpen: onMenuOpen, onMenuClose: onMenuClose, menuPlacement: menuPlacement, filterOption: filterOption })));
});
function required$9() {
    return (value) => {
        if (value === null)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const SelectValidators = {
    required: required$9,
};

/**
 * A validated dropdown component based on `react-select` that allows selecting
 * multiple options.
 *
 * If any options have isFixed: true, you should sort the options so that fixed options
 * come before unfixed. Sorting the options in the component would cause poor performance
 * when there are many options and the options array is not referentially stable.
 *
 * This component is expensive to render so use `React.memo` when necessary.
 */
const ValidatedMultiSelect = React__default["default"].memo(({ id, name, options, validators, showValidation, placeholder, className, formControlSize, width, components, isLoading, enabled = defaultSelectProps.enabled, isClearable = defaultSelectProps.isClearable, getStyles = defaultSelectProps.getStyles, isOptionEnabled, menuIsOpen, onMenuOpen, onMenuClose, menuPlacement = defaultSelectProps.menuPlacement, ...props }) => {
    const nonGroupOptions = getNonGroupOptions(options);
    const { value, onChange: _onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: [],
    });
    function onChange(options0, { action, removedValue, }) {
        let options;
        switch (action) {
            case 'clear':
                options = nonGroupOptions.filter((o) => o.isFixed);
                break;
            case 'remove-value':
            case 'pop-value':
                if (removedValue && removedValue.isFixed)
                    return;
                options = options0 ? options0 : [];
                break;
            default:
                options = options0 ? options0 : [];
                break;
        }
        options = lodash.sortBy(options, (o) => !o.isFixed);
        const newValue = options.map((o) => o.value);
        _onChange(newValue);
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators,
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    const { themeColors } = React.useContext(ItiReactContext);
    const optionMap = new Map(nonGroupOptions.map((o) => [o.value, o]));
    // Order of the `value` array determines the order the selected options are displayed in.
    // This way, when a new option is added, it is appended to the selected options instead of
    // potentaily being inserted in the middle
    const selectedOptions = value
        .map((v) => optionMap.get(v))
        .filter((o) => !!o);
    let isOptionDisabled;
    if (isOptionEnabled)
        isOptionDisabled = (o) => !isOptionEnabled(o);
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement(Select__default["default"], { inputId: id, name: name, className: className, options: options, placeholder: placeholder, value: selectedOptions, onChange: onChange, isClearable: isClearable, isDisabled: !enabled, isLoading: isLoading, isOptionDisabled: isOptionDisabled, styles: getStyles({
                valid: !validatorOutput,
                showValidation,
                themeColors,
                width,
                formControlSize,
                isMulti: true,
            }), "aria-label": props['aria-label'], "aria-labelledby": props['aria-labelledby'], 
            // eslint-disable-next-line -- TODO:SAM
            components: components, isMulti: true, menuIsOpen: menuIsOpen, onMenuOpen: onMenuOpen, onMenuClose: onMenuClose, menuPlacement: menuPlacement, filterOption: filterOption })));
});
function required$8() {
    return (value) => {
        if (value.length === 0)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const MultiSelectValidators = {
    required: required$8,
};

/**
 * A validated dropdown component based on `react-select` that loads its
 * options via an API call.
 *
 * This component is expensive to render so use `React.memo` when necessary.
 */
const ValidatedAsyncSelect = React__default["default"].memo(({ id, name, validators, showValidation, loadOptions, placeholder, noOptionsMessage, className, formControlSize, width, components, isLoading, enabled = defaultSelectProps.enabled, isClearable = defaultSelectProps.isClearable, getStyles = defaultSelectProps.getStyles, isOptionEnabled, menuIsOpen, onMenuOpen, onMenuClose, menuPlacement = defaultSelectProps.menuPlacement, ...props }) => {
    const { value, onChange: _onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: null,
    });
    function onChange(option0, actionMeta) {
        // option will be an array if the user presses backspace
        // This is so that if isClearable = false, null will never be passed to the
        // onChange prop
        if (!isClearable && actionMeta.action === 'pop-value')
            return;
        const option = option0;
        // Be careful with the conditional - option.value could be 0
        let newValue = null;
        if (option && option.value !== null) {
            newValue = option;
        }
        _onChange(newValue);
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators,
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    const { themeColors } = React.useContext(ItiReactContext);
    const stylesOptions = {
        valid: !validatorOutput,
        showValidation,
        themeColors,
        width,
        formControlSize,
        isMulti: false,
    };
    let isOptionDisabled;
    if (isOptionEnabled)
        isOptionDisabled = (o) => !isOptionEnabled(o);
    const loadOptionsRef = React.useRef(loadOptions);
    React.useEffect(() => {
        loadOptionsRef.current = loadOptions;
    });
    const loadOptionsDebouncedRef = React.useRef(debounce__default["default"]((inputValue) => loadOptionsRef.current(inputValue), 500));
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement(AsyncSelect__default["default"], { name: name, className: className, inputId: id, loadOptions: loadOptionsDebouncedRef.current, value: value, noOptionsMessage: noOptionsMessage, placeholder: placeholder, onChange: onChange, isClearable: isClearable, isDisabled: !enabled, isLoading: isLoading, isOptionDisabled: isOptionDisabled, styles: getStyles(stylesOptions), "aria-label": props['aria-label'], "aria-labelledby": props['aria-labelledby'], 
            // eslint-disable-next-line -- TODO:SAM
            components: components, menuIsOpen: menuIsOpen, onMenuOpen: onMenuOpen, onMenuClose: onMenuClose, menuPlacement: menuPlacement, filterOption: filterOption })));
});
function required$7() {
    return (value) => {
        if (value === null)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const AsyncSelectValidators = {
    required: required$7,
};

const AsyncSelectUtil = {
    /** @returns e.g. "Search users..." */
    getPlaceholder: (entityNamePlural) => `Search ${entityNamePlural}...`,
    /** @returns e.g. "No users matched your search" or "Begin typing to see matching users" */
    getNoOptionsMessage: (entityNamePlural) => ({ inputValue }) => inputValue
        ? `No ${entityNamePlural} matched your search`
        : `Begin typing to see matching ${entityNamePlural}`,
};

/**
 * A quick way to create an element that looks like a link but behaves
 * like a button.
 */
function LinkButton({ children, onClick, ...passThroughProps }) {
    /* eslint-disable jsx-a11y/anchor-is-valid */
    return (React__default["default"].createElement("a", { ...passThroughProps, href: "#", onClick: (e) => {
            e.preventDefault();
            if (onClick)
                onClick(e);
        }, role: "button" }, children));
}

const defaultTimeInputValue = {
    hours: undefined,
    minutes: undefined,
    ampm: undefined,
};
/**
 * Example:
 *
 * ```
 * timeInputValueFromDecimalHours(9.75)
 * // returns { hours: 9, minutes: 45, ampm: 'am' }
 * ```
 *
 * @returns a [[`TimeInputValue`]] or `undefined` if `decimalHours` is `undefined`.
 */
function timeInputValueFromDecimalHours(decimalHours) {
    if (typeof decimalHours === 'undefined')
        return defaultTimeInputValue;
    const { hours, minutes } = itiReactCore.toHoursAndMinutes(decimalHours);
    const d = moment__default["default"]().hour(hours).minute(minutes);
    return {
        hours: parseInt(d.format('h')),
        minutes: d.minute(),
        ampm: d.format('a'),
    };
}
function timeInputValueToDecimalHours(value) {
    if (typeof value.hours === 'undefined' ||
        typeof value.minutes === 'undefined' ||
        typeof value.ampm === 'undefined') {
        return undefined;
    }
    let hours = value.hours % 12;
    if (value.ampm === 'pm')
        hours += 12;
    return itiReactCore.toDecimalHours(hours, value.minutes);
}
//
// Supporting functions for component
//
const basicValidator = (value) => {
    const { hours, minutes, ampm } = value;
    const types = [typeof hours, typeof minutes, typeof ampm];
    let undefinedCount = 0;
    for (const type of types) {
        if (type === 'undefined')
            undefinedCount += 1;
    }
    if (undefinedCount !== 0 && undefinedCount !== 3)
        return 'You must enter a valid time or leave all fields blank.';
    return undefined;
};
const toOption = (x) => ({ value: x, label: x.toString() });
const options$1 = {
    hours: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12].map(toOption),
    minutes: ['00', '15', '30', '45'].map((m) => ({ value: parseInt(m), label: m })),
    ampm: ['am', 'pm'].map(toOption),
};
/**
 * The default clear button used in [[`TimeInput`]]. You can provide a different
 * clear button component if you want.
 */
function DefaultClearButtonComponent({ onClick, enabled, }) {
    if (!enabled) {
        return React__default["default"].createElement("span", { className: "default-clear-button disabled" }, "Clear");
    }
    return (React__default["default"].createElement(LinkButton, { onClick: onClick, className: "default-clear-button" }, "Clear"));
}
function fromSelectValue(selectValue) {
    if (selectValue === '' || selectValue === null)
        return undefined;
    return selectValue;
}
/**
 * A time of day input with dropdowns for hours, minutes, and am/pm.
 *
 * Make sure to pass `individualInputsRequired=true` and `isClearable=false`
 * as well as `TimeValidators.required()` if the time is required.
 *
 * It's not the most performant or user-friendly component but it has gotten the job
 * done so far.
 */
const TimeInput = React__default["default"].memo(({ showValidation, enabled = true, name, isClearable = true, clearButtonComponent: ClearButton = DefaultClearButtonComponent, individualInputsRequired, ...otherProps }) => {
    const { value, onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: defaultTimeInputValue,
    });
    function onHoursChange(selectValue) {
        const hours = fromSelectValue(selectValue);
        if (typeof hours !== 'number')
            throw new Error('Hours is not a number.');
        onChange({
            ...value,
            hours,
        });
    }
    function onMinutesChange(selectValue) {
        const minutes = fromSelectValue(selectValue);
        if (typeof minutes !== 'number')
            throw new Error('Hours is not a number.');
        onChange({
            ...value,
            minutes,
        });
    }
    function onAmpmChange(selectValue) {
        const ampm = fromSelectValue(selectValue);
        if (!(ampm === 'am' || ampm === 'pm' || typeof ampm === 'undefined'))
            throw new Error('ampm is not am, pm , or undefined.');
        onChange({
            ...value,
            ampm,
        });
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: [basicValidator, ...otherProps.validators],
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const { hours, minutes, ampm } = value;
    const indiviudalInputValidators = [];
    if (individualInputsRequired) {
        // don't display any feedback under individual fields
        indiviudalInputValidators.push((value) => value === null ? itiReactCore.INVALID_NO_FEEDBACK : undefined);
    }
    const commonProps = {
        showValidation,
        validators: indiviudalInputValidators,
        enabled,
    };
    return (React__default["default"].createElement("div", { className: "time-input" },
        React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
            React__default["default"].createElement("div", { className: "flex-container" },
                React__default["default"].createElement("div", { className: "input" },
                    React__default["default"].createElement(ValidatedSelect, { ...commonProps, name: `${name}_hours`, value: itiReactCore.undefinedToNull(hours), onChange: onHoursChange, options: options$1.hours, placeholder: "HH", "aria-label": "Hours" })),
                React__default["default"].createElement("div", { className: "input" },
                    React__default["default"].createElement(ValidatedSelect, { ...commonProps, name: `${name}_minutes`, value: itiReactCore.undefinedToNull(minutes), onChange: onMinutesChange, options: options$1.minutes, placeholder: "mm", "aria-label": "Minutes" })),
                React__default["default"].createElement("div", { className: "input" },
                    React__default["default"].createElement(ValidatedSelect, { ...commonProps, name: `${name}_ampm`, value: itiReactCore.undefinedToNull(ampm), onChange: onAmpmChange, options: options$1.ampm, placeholder: "", "aria-label": "AM or PM" })),
                isClearable && !lodash.isEqual(value, defaultTimeInputValue) && (React__default["default"].createElement(ClearButton, { onClick: () => onChange(defaultTimeInputValue), enabled: enabled }))))));
});
function required$6() {
    return (value) => {
        const { hours, minutes, ampm } = value;
        if (typeof hours === 'undefined' ||
            typeof minutes === 'undefined' ||
            typeof ampm === 'undefined')
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
/** Validators for use with [[`TimeInput`]]. */
const TimeValidators = {
    required: required$6,
};

/** @internal */
function RadioButton(props) {
    const { name, value, enabled, radioOption, onChange, inline } = props;
    const id = `${name}-${radioOption.value}`;
    const classes = ['form-check', radioOption.value.toString()];
    if (inline)
        classes.push('form-check-inline');
    return (React__default["default"].createElement("div", { className: classes.join(' '), key: radioOption.value },
        React__default["default"].createElement("input", { type: "radio", className: "form-check-input", name: name, id: id, value: radioOption.value, checked: radioOption.value === value, onChange: () => onChange(radioOption.value), disabled: !enabled }),
        React__default["default"].createElement("label", { className: "form-check-label user-select-none", htmlFor: id }, radioOption.label)));
}

const classSeparator = '__';
/**
 * Radio button input. It renders a group of radio buttons based on the `options` you
 * pass in.
 *
 * You can customize the style of both the container and the individual radio buttons
 * / labels. Use the devtools element inspector to see the class names you need to
 * target.
 */
const RadioInput = React__default["default"].memo(({ options, enabled = true, showValidation, name, ...props }) => {
    const { value, onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: null,
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators: props.validators,
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    const buttonOptions = {
        inline: true,
        ...props.buttonOptions,
    };
    const containerClass = 'radio-button-container';
    const containerClasses = [containerClass, name + classSeparator + containerClass];
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement("div", { className: containerClasses.join(' ') }, options.map((o) => (React__default["default"].createElement(RadioButton, { radioOption: o, name: name, value: value, onChange: onChange, enabled: enabled, inline: buttonOptions.inline, key: o.value }))))));
});
function required$5() {
    return (value) => {
        if (value === null)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const RadioValidators = {
    required: required$5,
};

function convertValue(value) {
    if (typeof value === 'undefined')
        return undefined;
    if (value === null)
        return null;
    return value.toString();
}
function convertValidator$1(validator) {
    return (value) => {
        const booleanValue = value !== null ? value === true.toString() : null;
        return validator(booleanValue);
    };
}
function convertAsyncValidator$1(validator) {
    return (value) => {
        const booleanValue = value !== null ? value === true.toString() : null;
        return validator(booleanValue);
    };
}
/**
 * Wrapper around [[`RadioInput`]] for when there are only two options, true and false.
 */
function BooleanRadioInput({ trueFirst = true, value, onChange, defaultValue, validators, labels = { false: 'No', true: 'Yes' }, asyncValidator, ...passThroughProps }) {
    const options = [
        { value: true.toString(), label: labels.true },
        { value: false.toString(), label: labels.false },
    ];
    if (!trueFirst)
        options.reverse();
    return (React__default["default"].createElement(RadioInput, { options: options, value: convertValue(value), defaultValue: convertValue(defaultValue), onChange: (value) => {
            if (onChange)
                onChange(value === true.toString());
        }, validators: validators.map(convertValidator$1), asyncValidator: asyncValidator ? convertAsyncValidator$1(asyncValidator) : undefined, ...passThroughProps }));
}
//
//
//
function required$4() {
    return (value) => {
        if (value === null)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const BooleanRadioValidators = {
    required: required$4,
};

/* Not including the zone abbreviation (e.g. EST / EDT ). This becomes a huge pain in
 * the ass since the abbreviation depends on the current date. */
const commonTimeZones = [
    {
        ianaTimeZone: 'America/New_York',
        displayName: 'Eastern Time',
    },
    {
        ianaTimeZone: 'America/Chicago',
        displayName: 'Central Time',
    },
    {
        ianaTimeZone: 'America/Denver',
        displayName: 'Mountain Time',
    },
    {
        ianaTimeZone: 'America/Los_Angeles',
        displayName: 'Pacific Time',
    },
];
const usOptions$1 = commonTimeZones.map((o) => ({
    value: o.ianaTimeZone,
    label: o.displayName,
}));
const advancedOptions = tzdb.getTimeZones()
    .map((o) => ({
    value: o.name,
    label: o.rawFormat,
}))
    .filter((o) => !usOptions$1.find((oo) => o.value === oo.value));
const options = [
    { label: 'US time zones', options: usOptions$1 },
    { label: 'All time zones', options: advancedOptions },
];
const allOptions = [...usOptions$1, ...advancedOptions];
const optionValueSet = new Set(allOptions.map((o) => o.value));
/**
 * @internal
 *
 * Because tzdb groups time zones, there isn't a SelectOption for every IANA time zone.
 * This function converts the value to a time zone that has an option.
 */
function resolveValue(value, onChange) {
    if (value && !optionValueSet.has(value)) {
        const timeZone = tzdb.getTimeZones().find((tz) => tz.group.includes(value));
        if (timeZone)
            onChange(timeZone.name);
    }
}
function convertValidator(validator) {
    return (value) => {
        if (typeof value === 'number')
            throw new Error('Time zone validator received a number.');
        return validator(value);
    };
}
function convertAsyncValidator(validator) {
    return (value) => {
        if (typeof value === 'number')
            throw new Error('Time zone async validator received a number.');
        return validator(value);
    };
}
/**
 * A time zone dropdown. The 4 common US time zones are shown first, followed by every
 * other time zone. It uses the `@vvo/tzdb` time zone database.
 */
function TimeZoneInput({ id, name, placeholder, formControlSize, isClearable, width, validators, showValidation, enabled = true, asyncValidator, onAsyncError, onValidChange, validationKey, ...props }) {
    const { value, onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: null,
    });
    const onChangeRef = React.useRef(onChange);
    React.useEffect(() => {
        onChangeRef.current = onChange;
    });
    React.useEffect(() => {
        resolveValue(value, onChangeRef.current);
    }, [value]);
    return (React__default["default"].createElement(ValidatedSelect, { id: id, name: name, options: options, validators: validators.map(convertValidator), asyncValidator: asyncValidator ? convertAsyncValidator(asyncValidator) : undefined, onAsyncError: onAsyncError, value: value, onChange: onChange, showValidation: showValidation, placeholder: placeholder, formControlSize: formControlSize, width: width, isClearable: isClearable, enabled: enabled, onValidChange: onValidChange, validationKey: validationKey, formLevelValidatorOutput: props.formLevelValidatorOutput }));
}
function required$3() {
    return (value) => {
        if (value === null)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
const TimeZoneValidators = {
    required: required$3,
};

/** A blank [[`AddressInputValue`]]. */
const defaultAddressInputValue = {
    line1: '',
    line2: '',
    city: '',
    state: '',
    postalCode: '',
};

const defaultPostalCodeValidationOptions = {
    allowCanadian: true,
};
/**
 * Determines if a postal code is valid. US 5-digit, US 9-digit, and Canadian postal
 * codes are supported.
 */
function isPostalCodeValid(code, options = defaultPostalCodeValidationOptions) {
    code = code.replace('-', '').replace(' ', '');
    if (code.length === 5 || code.length === 9) {
        // US zip codes - valid if digits only
        return /^\d*$/.test(code);
    }
    if (code.length === 6 && options.allowCanadian) {
        // Canadian postal codes, e.g. A1A 1A1
        return /^(?:[a-z]\d){3}$/i.test(code);
    }
    return false;
}
/**
 * Validator for a postal code input. Uses [[`isPostalCodeValid`]].
 */
function postalCodeValidator(options = defaultPostalCodeValidationOptions) {
    return (value) => {
        if (!isPostalCodeValid(value, options))
            return `Invalid ${options.allowCanadian ? 'postal' : 'zip'} code.`;
        return undefined;
    };
}

/**
 * Only use this if making your own address input.
 */
function disallowPartialAddress() {
    return (v) => {
        const requiredValues = [v.line1, v.city, v.state, v.postalCode];
        const enteredRequiredValues = requiredValues.filter((v) => !!v).length;
        if (enteredRequiredValues !== 0 &&
            enteredRequiredValues !== requiredValues.length) {
            return ('Partial addresses are not allowed. ' +
                'Fill out all required fields or leave the address empty.');
        }
        return undefined;
    };
}
/**
 * Only use this if making your own address input.
 */
function allFieldsValid(postalCodeValidationOptions) {
    return (value) => {
        // postalCode is the only field that has validation logic
        if (postalCodeValidator(postalCodeValidationOptions)(value.postalCode))
            return itiReactCore.INVALID_NO_FEEDBACK;
        return undefined;
    };
}
/**
 * Only use this if making your own address input.
 */
function allFieldLengthsValid(fieldLengths) {
    return (value) => {
        if (value.line1.length > fieldLengths.line1 ||
            value.line2.length > fieldLengths.line2 ||
            value.city.length > fieldLengths.city) {
            return itiReactCore.INVALID_NO_FEEDBACK;
        }
        return undefined;
    };
}
function required$2() {
    return (value) => {
        if (!(value.line1 && value.city && value.state && value.postalCode)) {
            // "This field is required." will be displayed under individual inputs
            return itiReactCore.INVALID_NO_FEEDBACK;
        }
        return undefined;
    };
}
/** Validators for use with [[`AddressInput`]]. */
const AddressValidators = {
    required: required$2,
};
/** Validators you can use to build your own custom address input. */
const InternalAddressValidators = {
    disallowPartialAddress,
    allFieldsValid,
    allFieldLengthsValid,
};

function statesFromObject(obj) {
    return Object.entries(obj).map(([abbr, name]) => ({ abbr, name }));
}
const usStates = statesFromObject({
    AL: 'Alabama',
    AK: 'Alaska',
    AS: 'American Samoa',
    AZ: 'Arizona',
    AR: 'Arkansas',
    CA: 'California',
    CO: 'Colorado',
    CT: 'Connecticut',
    DE: 'Delaware',
    DC: 'District of Columbia',
    FL: 'Florida',
    FM: 'Micronesia',
    GA: 'Georgia',
    GU: 'Guam',
    HI: 'Hawaii',
    ID: 'Idaho',
    IL: 'Illinois',
    IN: 'Indiana',
    IA: 'Iowa',
    KS: 'Kansas',
    KY: 'Kentucky',
    LA: 'Louisiana',
    ME: 'Maine',
    MH: 'Marshall Islands',
    MD: 'Maryland',
    MA: 'Massachusetts',
    MI: 'Michigan',
    MN: 'Minnesota',
    MS: 'Mississippi',
    MO: 'Missouri',
    MT: 'Montana',
    NC: 'North Carolina',
    MP: 'Northern Mariana Islands',
    NE: 'Nebraska',
    NV: 'Nevada',
    NH: 'New Hampshire',
    NJ: 'New Jersey',
    NM: 'New Mexico',
    NY: 'New York',
    ND: 'North Dakota',
    OH: 'Ohio',
    OK: 'Oklahoma',
    OR: 'Oregon',
    PW: 'Palau',
    PA: 'Pennsylvania',
    PR: 'Puerto Rico',
    RI: 'Rhode Island',
    SC: 'South Carolina',
    SD: 'South Dakota',
    TN: 'Tennessee',
    TX: 'Texas',
    UT: 'Utah',
    VA: 'Virginia',
    VI: 'US Virgin Islands',
    VT: 'Vermont',
    WA: 'Washington',
    WV: 'West Virginia',
    WI: 'Wisconsin',
    WY: 'Wyoming',
});
const canadianProvinces = statesFromObject({
    AB: 'Alberta',
    BC: 'British Columbia',
    MB: 'Manitoba',
    NB: 'New Brunswick',
    NL: 'Newfoundland and Labrador',
    NS: 'Nova Scotia',
    NT: 'Northwest Territories',
    NU: 'Nunavut',
    ON: 'Ontario',
    PE: 'Prince Edward Island',
    QC: 'Québec',
    SK: 'Saskatchewan',
    YT: 'Yukon',
});
/** Get a list of US states and/or Canadian provinces. */
function getStates(options) {
    let states = [];
    if (options.includeUsStates)
        states = [...states, ...usStates];
    if (options.includeCanadianProvinces)
        states = [...states, ...canadianProvinces];
    return states;
}
function stateToOption(state) {
    return { value: state.abbr, label: state.abbr };
}
const usOptions = usStates.map(stateToOption);
const canadianOptions = canadianProvinces.map(stateToOption);
/** Get a list of [[`SelectOption`]]s for US states and/or Canadian provinces. */
function getStateOptions(options) {
    if (options.includeUsStates && options.includeCanadianProvinces) {
        return [
            { label: 'US States', options: usOptions },
            { label: 'Canadian Provinces', options: canadianOptions },
        ];
    }
    if (options.includeUsStates) {
        return usOptions;
    }
    if (options.includeCanadianProvinces) {
        return canadianOptions;
    }
    return [];
}

const ValidatedInput = React__default["default"].memo(({ id, type = 'text', showValidation, enabled = true, name, ...otherProps }) => {
    const { value, onChange: _onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: '',
    });
    function onChange(e) {
        _onChange(e.currentTarget.value);
    }
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: otherProps.validators,
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const classes = [
        getValidationClass(!validatorOutput, showValidation),
        'form-control',
    ];
    if (otherProps.className)
        classes.push(otherProps.className);
    const inputAttributes = { ...otherProps.inputAttributes, disabled: !enabled };
    let input;
    if (type && type.toLowerCase() === 'textarea') {
        input = (React__default["default"].createElement("textarea", { id: id, name: name, className: classes.join(' '), value: value, onChange: onChange, ...inputAttributes }));
    }
    else {
        input = (React__default["default"].createElement("input", { id: id, name: name, type: type, className: classes.join(' '), value: value, onChange: onChange, ...inputAttributes }));
    }
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation }, input));
});

/**
 * A validated address input. Supports US and Canadian addresses.
 *
 * Field lengths and the default value of `allowCanadian` are configured via
 * [[`ItiReactContext`]].
 */
const AddressInput = React__default["default"].memo(({ showValidation, getStateSelectStyles, individualInputsRequired, enabled = true, ...otherProps }) => {
    var _a;
    const context = React.useContext(ItiReactContext);
    const fieldLengths = context.fieldLengths.address;
    const allowCanadian = (_a = otherProps.allowCanadian) !== null && _a !== void 0 ? _a : context.addressInput.allowCanadian;
    const validators = [
        ...otherProps.validators,
        InternalAddressValidators.allFieldsValid({ allowCanadian }),
        InternalAddressValidators.allFieldLengthsValid(fieldLengths),
    ];
    if (!individualInputsRequired) {
        // this validator is only needed for optional addresses
        validators.push(InternalAddressValidators.disallowPartialAddress());
    }
    const fieldValidators = React.useMemo(() => {
        const baseFieldValidators = [];
        const stateValidators = [];
        if (individualInputsRequired) {
            baseFieldValidators.push(itiReactCore.Validators.required());
            stateValidators.push(SelectValidators.required());
        }
        return {
            line1: [...baseFieldValidators, itiReactCore.Validators.maxLength(fieldLengths.line1)],
            line2: [itiReactCore.Validators.maxLength(fieldLengths.line2)],
            city: [...baseFieldValidators, itiReactCore.Validators.maxLength(fieldLengths.city)],
            state: stateValidators,
            postalCode: [
                ...baseFieldValidators,
                postalCodeValidator({ allowCanadian }),
            ],
        };
    }, [individualInputsRequired, fieldLengths, allowCanadian]);
    const { value, onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: defaultAddressInputValue,
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name: otherProps.name,
        onValidChange: otherProps.onValidChange,
        validators,
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const stateOptions = React.useMemo(() => getStateOptions({
        includeUsStates: true,
        includeCanadianProvinces: allowCanadian,
    }), [allowCanadian]);
    const vProps = {
        showValidation,
        validationKey: individualInputsRequired.toString(),
    };
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement("div", { className: "address-input" },
            React__default["default"].createElement("div", { className: "address-row address-row-1" },
                React__default["default"].createElement(ValidatedInput, { name: "line1", value: value.line1, onChange: (line1) => onChange({ ...value, line1 }), validators: fieldValidators.line1, inputAttributes: {
                        placeholder: 'Line 1',
                        'aria-label': 'Address line 1',
                    }, enabled: enabled, ...vProps })),
            React__default["default"].createElement("div", { className: "address-row address-row-2" },
                React__default["default"].createElement(ValidatedInput, { name: "line2", value: value.line2, onChange: (line2) => onChange({ ...value, line2 }), validators: fieldValidators.line2, inputAttributes: {
                        placeholder: 'Line 2',
                        'aria-label': 'Address line 2',
                    }, enabled: enabled, ...vProps })),
            React__default["default"].createElement("div", { className: "address-row address-row-3" },
                React__default["default"].createElement("div", { className: "city-input-container" },
                    React__default["default"].createElement(ValidatedInput, { name: "city", value: value.city, onChange: (city) => onChange({ ...value, city }), validators: fieldValidators.city, inputAttributes: {
                            placeholder: 'City',
                            'aria-label': 'City',
                        }, enabled: enabled, ...vProps })),
                React__default["default"].createElement("div", { className: "state-select" },
                    React__default["default"].createElement(ValidatedSelect, { name: "state", value: value.state ? value.state.toUpperCase() : null, onChange: React.useCallback((state) => onChange({
                            ...value,
                            state: state !== null ? state : '',
                        }), [value, onChange]), options: stateOptions, width: 115, placeholder: "State", validators: fieldValidators.state, isClearable: !individualInputsRequired, "aria-label": "State", enabled: enabled, getStyles: getStateSelectStyles, ...vProps })),
                React__default["default"].createElement(ValidatedInput, { name: "postalCode", value: value.postalCode, onChange: (postalCode) => onChange({ ...value, postalCode }), validators: fieldValidators.postalCode, inputAttributes: {
                        placeholder: 'ZIP',
                        'aria-label': allowCanadian ? 'Postal code' : 'ZIP',
                    }, enabled: enabled, ...vProps })))));
});

const defaultPersonNameInputValue = {
    first: '',
    middle: '',
    last: '',
};
const noPartialNamesValidator = (value) => {
    const allSpecified = !!(value.first && value.last);
    const allBlank = !value.first && !value.middle && !value.last;
    if (!allBlank && !allSpecified)
        return 'Both first and last name must be specified, or all fields must be left blank.';
    return undefined;
};
function required$1() {
    return (value) => {
        if (!(value.first && value.last))
            return 'First and last name are required.';
        return undefined;
    };
}
/** Validators for use with [[`PersonNameInput`]]. */
const PersonNameValidators = {
    required: required$1,
};
/**
 * A person name input that displays inputs for first name, middle name (optional), and
 * last name.
 *
 * If we just want first and last name, we will often just use two normal [[`ValidatedInputs`]]
 * instead of `PersonNameInput`.
 */
const PersonNameInput = React__default["default"].memo(({ name, showValidation, individualInputsRequired, showMiddleNameInput = false, fluid = false, inputAttributesMap: propsInputAttributesMap = {}, enabledInputs = ['first', 'middle', 'last'], ...otherProps }) => {
    const { value, onChange } = itiReactCore.useControlledValue({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: defaultPersonNameInputValue,
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: [...otherProps.validators, noPartialNamesValidator],
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const inputAttributesMap = lodash.defaults({ ...propsInputAttributesMap }, {
        prefix: {},
        first: {},
        middle: {},
        last: {},
    });
    const classes = ['person-name-input'];
    if (fluid)
        classes.push('person-name-input-fluid');
    const { onChildValidChange, allFieldsValid } = itiReactCore.useFieldValidity();
    const { fieldLengths } = React.useContext(ItiReactContext);
    const firstNameValidators = [itiReactCore.Validators.maxLength(fieldLengths.personName.first)];
    const lastNameValidators = [itiReactCore.Validators.maxLength(fieldLengths.personName.last)];
    if (individualInputsRequired) {
        firstNameValidators.unshift(itiReactCore.Validators.required());
        lastNameValidators.unshift(itiReactCore.Validators.required());
    }
    const vProps = { showValidation, onValidChange: onChildValidChange };
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation && allFieldsValid },
        React__default["default"].createElement("div", { className: classes.join(' ') },
            React__default["default"].createElement(ValidatedInput, { name: `${name}First`, inputAttributes: {
                    placeholder: 'First',
                    'aria-label': 'First name',
                    ...inputAttributesMap.first,
                }, enabled: enabledInputs.includes('first'), value: value.first, onChange: (first) => onChange({ ...value, first }), validators: firstNameValidators, ...vProps }),
            showMiddleNameInput && (React__default["default"].createElement(ValidatedInput, { name: `${name}Middle`, inputAttributes: {
                    placeholder: 'Middle',
                    'aria-label': 'Middle name',
                    ...inputAttributesMap.middle,
                }, enabled: enabledInputs.includes('middle'), value: value.middle, onChange: (middle) => onChange({ ...value, middle }), 
                // Never required because some people don't have middle names
                validators: [
                    itiReactCore.Validators.maxLength(fieldLengths.personName.middle),
                ], ...vProps })),
            React__default["default"].createElement(ValidatedInput, { name: `${name}Last`, inputAttributes: {
                    placeholder: 'Last',
                    'aria-label': 'Last name',
                    ...inputAttributesMap.last,
                }, enabled: enabledInputs.includes('last'), value: value.last, onChange: (last) => onChange({ ...value, last }), validators: lastNameValidators, ...vProps }))));
});

/**
 * A file upload control that can be made a required field.
 */
function FileInput({ id, accept, enabled, inputAttributes, showValidation, name, ...props }) {
    const { value, onChange } = itiReactCore.useControlledValue({
        value: props.value,
        onChange: props.onChange,
        defaultValue: props.defaultValue,
        fallbackValue: null,
    });
    const validatorOutput = itiReactCore.useValidation({
        value,
        name,
        onValidChange: props.onValidChange,
        validators: props.validators,
        validationKey: props.validationKey,
        asyncValidator: props.asyncValidator,
        onAsyncError: props.onAsyncError,
        formLevelValidatorOutput: props.formLevelValidatorOutput,
    });
    return (React__default["default"].createElement(ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation },
        React__default["default"].createElement("input", { id: id, type: "file", name: name, accept: accept, onChange: (e) => {
                var _a;
                const files = (_a = e.target) === null || _a === void 0 ? void 0 : _a.files;
                if (files && files[0]) {
                    onChange(files[0]);
                }
                else {
                    onChange(null);
                }
            } })));
}
function required() {
    return (value) => {
        if (!value)
            return itiReactCore.Validators.required()('');
        return undefined;
    };
}
/** For use with [[`FileInput`]]. */
const FileValidators = { required };

// Mark as disabled (rather than making it invisible) so the pager doesn't
// jump around
function PagerLink({ active, onClick, enabled, children }) {
    return (React__default["default"].createElement("li", { className: `page-item ${active ? 'active' : ''} ${enabled ? '' : 'disabled'}` },
        React__default["default"].createElement(LinkButton, { className: "page-link", tabIndex: enabled ? undefined : -1, onClick: onClick }, children)));
}
/**
 * A simple pagination control that does not allow setting the number of items on a page.
 *
 * These days, [[`ConfigurablePager`]] is generally preferred.
 */
const Pager = React__default["default"].memo(({ page, totalPages, onPageChange, enabled = true, containerClassName = 'pagination-container', }) => {
    const firstPage = 1;
    const hasPrevious = page !== firstPage;
    const hasNext = page < totalPages;
    // Don't want to show too many pages if there are a lot
    const pagesToDisplay = 5; // odd number
    let pageNumbers = [page];
    let distance = 1;
    // add page number to the left and right until hit the pagesToDisplay
    // and/or run out of pages
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const left = page - distance;
        const addLeft = left >= firstPage;
        if (addLeft)
            pageNumbers = [left].concat(pageNumbers);
        const right = page + distance;
        const addRight = right <= totalPages;
        if (addRight)
            pageNumbers.push(right);
        if (pageNumbers.length === pagesToDisplay || (!addLeft && !addRight)) {
            break;
        }
        distance += 1;
    }
    return (React__default["default"].createElement("nav", { "aria-label": "Page navigation", className: containerClassName },
        React__default["default"].createElement("ul", { className: "pagination" },
            React__default["default"].createElement(PagerLink, { onClick: () => onPageChange(page - 1), key: "prev", enabled: enabled && hasPrevious },
                React__default["default"].createElement("span", { "aria-hidden": "true" }, "\u00AB"),
                React__default["default"].createElement("span", { className: "visually-hidden" }, "Previous")),
            pageNumbers.map((i) => (React__default["default"].createElement(PagerLink, { onClick: () => onPageChange(i), active: page === i, key: i.toString(), enabled: enabled }, i))),
            React__default["default"].createElement(PagerLink, { onClick: () => onPageChange(page + 1), key: "next", enabled: enabled && hasNext },
                React__default["default"].createElement("span", { "aria-hidden": "true" }, "\u00BB"),
                React__default["default"].createElement("span", { className: "visually-hidden" }, "Next")))));
});

/* eslint-disable react/button-has-type */
/**
 * Submit button/link that displays a loading indicator and disables the onClick handler
 * when `submitting=true`.
 */
function SubmitButton({ element = 'button', submitting, children, onClick, onClickEnabled = true, enabled = true, className, type = 'submit', ...passThroughProps }) {
    const { renderLoadingIndicator } = React.useContext(ItiReactContext);
    if (submitting || !enabled) {
        onClickEnabled = false;
    }
    if (!onClickEnabled) {
        onClick = undefined;
    }
    if (typeof className === 'undefined') {
        className = '';
    }
    className += ' submit-button';
    if (element === 'button') {
        // if submitting or !onClickEnabled, set type to 'button' to prevent
        // buttons with type="submit" submitting the form
        const _type = onClickEnabled && !submitting ? type : 'button';
        return (React__default["default"].createElement("button", { ...passThroughProps, className: className, onClick: onClick, type: _type, disabled: !enabled },
            submitting ? React__default["default"].createElement("span", { className: "hidden-label" }, children) : children,
            submitting && (React__default["default"].createElement("div", { className: "loading-icon-container" }, renderLoadingIndicator()))));
    }
    if (!enabled)
        className += ' disabled';
    return (React__default["default"].createElement(LinkButton, { ...passThroughProps, className: className, onClick: onClick },
        children,
        submitting && React__default["default"].createElement("span", null,
            " ",
            renderLoadingIndicator())));
}

/**
 * A [[`Dialog`]] that has an action button and a cancel button.
 */
function ActionDialog({ actionButtonText, actionButtonClass = 'btn-primary', cancelButtonText = 'Cancel', action, actionInProgress, title, modalClass, onOpen, onClose, children, focusFirst, focusFirstOptions, actionButtonEnabled = true, showFooter = true, onCancel, closeRef, }) {
    let footer;
    if (showFooter) {
        footer = (React__default["default"].createElement(React__default["default"].Fragment, null,
            React__default["default"].createElement(SubmitButton, { type: "button", onClick: action, className: `btn ${actionButtonClass !== null && actionButtonClass !== void 0 ? actionButtonClass : ''}`, submitting: actionInProgress, enabled: actionButtonEnabled }, actionButtonText),
            React__default["default"].createElement("button", { type: "button", className: "btn btn-secondary", "data-bs-dismiss": actionInProgress || onCancel ? '' : 'modal', onClick: onCancel }, cancelButtonText)));
    }
    return (React__default["default"].createElement(Dialog, { title: title, closeRef: closeRef, modalClass: modalClass, modalFooter: footer, onOpen: onOpen, onClose: onClose, focusFirst: focusFirst, focusFirstOptions: focusFirstOptions, allowDismiss: !actionInProgress }, children));
}
/**
 * Wrapper around the Bootstrap dialog component.
 *
 * - **DO NOT** unmount the dialog except in the `onClose` function. This will cause
 *   the dialog to be abruptly removed without an animation.
 * - Use `closeRef` to close the dialog from the parent component.
 * - The first input element is automatically focused when the dialog opens. This can be
 *   controlled with the `focusFirst` and `focusFirstOptions` props.
 */
function Dialog({ title, onOpen, onClose, modalFooter, children, closeRef, modalClass = '', focusFirst = true, allowDismiss = true, ...otherProps }) {
    const focusFirstOptions = { additionalTagNames: [], ...otherProps.focusFirstOptions };
    const elementRef = React.useRef(null);
    const modalRef = React.useRef();
    React.useEffect(() => {
        if (closeRef) {
            closeRef.current = () => {
                if (modalRef.current)
                    modalRef.current.hide();
            };
        }
    });
    useEventListener__default["default"]('keydown', (e) => {
        if (e.key === 'Escape' && allowDismiss && elementRef.current) {
            if (modalRef.current)
                modalRef.current.hide();
        }
    });
    // dependencies = [] because this effect should only run once. By design, a Dialog
    // instance can only be opened once.
    /* eslint-disable react-hooks/exhaustive-deps */
    React.useEffect(() => {
        const el = elementRef.current;
        if (!el)
            throw new Error('modal element ref is not initialized.');
        // keyboard: false because we handle closing the modal when Escape is pressed ourselves
        modalRef.current = new bootstrap__namespace.Modal(el, {
            backdrop: 'static',
            keyboard: false,
        });
        modalRef.current.show();
        if (!focusFirst)
            return;
        // Focus the first field. autofocus attribute does not work in Bootstrap modals
        function shownHandler() {
            if (!el)
                throw new Error('el is null. Should be impossible.');
            const selector = [
                'input',
                'select',
                'textarea',
                ...focusFirstOptions.additionalTagNames,
            ].join(', ');
            const candidates = Array.from(el.querySelectorAll(selector));
            for (const candidate of candidates) {
                if (candidate.classList.contains('btn-close'))
                    continue;
                if (candidate.hasAttribute('readonly'))
                    continue;
                if (candidate.hasAttribute('disabled'))
                    continue;
                if (candidate.getAttribute('type') === 'hidden')
                    continue;
                candidate.focus();
                break;
            }
        }
        el.addEventListener('shown.bs.modal', shownHandler);
        return () => {
            el.removeEventListener('shown.bs.modal', shownHandler);
        };
    }, []);
    /* eslint-enable react-hooks/exhaustive-deps */
    const onOpenRef = React.useRef(onOpen);
    const onCloseRef = React.useRef(onClose);
    React.useEffect(() => {
        onOpenRef.current = onOpen;
        onCloseRef.current = onClose;
    });
    React.useEffect(() => {
        if (!elementRef.current)
            throw new Error('modal element ref is not initialized.');
        elementRef.current.addEventListener('shown.bs.modal', () => {
            if (onOpenRef.current)
                onOpenRef.current();
        });
        elementRef.current.addEventListener('hidden.bs.modal', () => {
            onCloseRef.current();
        });
    }, []);
    return (React__default["default"].createElement("div", { ref: elementRef, className: "modal fade", tabIndex: -1, role: "dialog" },
        React__default["default"].createElement("div", { className: `modal-dialog ${modalClass}`, role: "document" },
            React__default["default"].createElement("div", { className: "modal-content" },
                React__default["default"].createElement("div", { className: "modal-header" },
                    React__default["default"].createElement("h5", { className: "modal-title" }, title),
                    React__default["default"].createElement("button", { type: "button", className: "btn-close", "data-bs-dismiss": allowDismiss ? 'modal' : undefined, "aria-label": "Close" })),
                React__default["default"].createElement("div", { className: "modal-body" }, children),
                modalFooter && React__default["default"].createElement("div", { className: "modal-footer" }, modalFooter)))));
}
function cleanupImproperlyClosedDialog() {
    document.body.classList.remove('modal-open');
    document.body.removeAttribute('style');
    const backdrop = document.querySelector('.modal-backdrop');
    if (backdrop && backdrop.parentElement) {
        backdrop.parentElement.removeChild(backdrop);
    }
}

/* eslint-disable */
function formToObject(form) {
    const array = form.serializeArray();
    const obj = {};
    for (const pair of array) {
        obj[pair.name] = pair.value;
    }
    // serializeArray() ignores checkbox if it's unchecked and puts its value as "on"
    // if it is checked. This doesn't play well with web API so here we turn the
    // checkboxes into booleans.
    const checkboxes = form.find('[type="checkbox"]').toArray();
    for (const checkboxEl of checkboxes) {
        const checkbox = $(checkboxEl);
        const name = checkbox.attr('name');
        if (name) {
            obj[name] = checkbox.is(':checked');
        }
    }
    return obj;
}
/**
 * A wrapper around [[`ActionDialog`]] that removes a lot of the boilerplate needed
 * for dialogs that contain a form.
 *
 * ```tsx
 * interface ExampleProps {
 *     onSuccess(responseData: number): Promise<void>
 *     onClose(): void
 * }
 *
 * export function Example({
 *     onSuccess,
 *     onClose,
 * }: ExampleProps): ReactElement {
 *     const { onChildValidChange, allFieldsValid } = useFieldValidity()
 *     const [showValidation, setShowValidation] = useState(false)
 *     const vProps = { showValidation, onValidChange: onChildValidChange }
 *
 *     const [myNumber, setMyNumber] = useState('')
 *
 *     async function submit() {
 *         await api.product.performOperation()
 *
 *         return {
 *             responseData: parseInt(myNumber),
 *         }
 *     }
 *
 *     return (
 *         <EasyFormDialog
 *             title="Enter a Number"
 *             submitButtonText="Submit"
 *             formIsValid={allFieldsValid}
 *             showValidation={showValidation}
 *             onShowValidationChange={setShowValidation}
 *             onSubmit={submit}
 *             onSuccess={onSuccess}
 *             onClose={onClose}
 *         >
 *             <FormGroup label="My number">
 *                 {(id) => (
 *                     <ValidatedInput
 *                         id={id}
 *                         name="myNumber"
 *                         validators={[Validators.required(), Validators.integer()]}
 *                         value={myNumber}
 *                         onChange={setMyNumber}
 *                         {...vProps}
 *                     />
 *                 )}
 *             </FormGroup>
 *         </EasyFormDialog>
 *     )
 * }
 * ```
 */
function EasyFormDialog({ title, submitButtonText, submitEnabled: propsSubmitEnabled = true, submitButtonClass, cancelButtonText, formIsValid, showValidation, onShowValidationChange, onSuccess, modalClass, focusFirst, onClose, onSubmit, onCancel, showFooter, children, closeRef = {
    current: lodash.noop,
}, }) {
    const submitEnabled = propsSubmitEnabled && itiReactCore.getSubmitEnabled(formIsValid, showValidation);
    const { onError } = React.useContext(itiReactCore.ItiReactCoreContext);
    const [submitting, setSubmitting] = React.useState(false);
    const submittedTimeRef = React.useRef();
    const formRef = React.useRef(null);
    async function submit() {
        var _a;
        if (!submitEnabled)
            return;
        if (submitting)
            return;
        onShowValidationChange(true);
        if (!formIsValid)
            return;
        // Prevent double submit when Ctrl+Enter is pressed in Firefox
        if (submittedTimeRef.current &&
            moment__default["default"]().diff(submittedTimeRef.current, 'ms') < 200) {
            return;
        }
        submittedTimeRef.current = moment__default["default"]();
        setSubmitting(true);
        let formData = {};
        // jQuery is an optional peer dependency.
        // We have to be careful not to depend on @types/jquery either.
        /* eslint-disable */
        const jq = window.jQuery;
        if (jq) {
            if (!formRef.current)
                throw new Error('formRef.current is null.');
            formData = formToObject(jq(formRef.current));
        }
        /* eslint-enable */
        try {
            // hack to allow onSubmit to return void
            const onSubmitReturnValue = (await onSubmit(formData));
            const shouldClose = (_a = onSubmitReturnValue === null || onSubmitReturnValue === void 0 ? void 0 : onSubmitReturnValue.shouldClose) !== null && _a !== void 0 ? _a : true;
            const responseData = onSubmitReturnValue === null || onSubmitReturnValue === void 0 ? void 0 : onSubmitReturnValue.responseData;
            if (shouldClose) {
                // onSuccess may be loading data, so wait for it to finish before hiding the modal
                // and setting submitting=false
                await onSuccess(responseData);
                closeRef.current();
            }
        }
        catch (e) {
            onError(e);
            return;
        }
        setSubmitting(false);
    }
    // Submit form on Ctrl+Enter - convenient when you are typing in a textarea
    useEventListener__default["default"]('keypress', (e) => {
        if (e.ctrlKey && e.code === 'Enter') {
            void submit();
        }
    });
    return (React__default["default"].createElement(ActionDialog, { closeRef: closeRef, title: title, actionButtonText: submitButtonText, actionButtonEnabled: submitEnabled, actionButtonClass: submitButtonClass, cancelButtonText: cancelButtonText, action: submit, actionInProgress: submitting, modalClass: modalClass, onClose: onClose, focusFirst: focusFirst, showFooter: showFooter, onCancel: onCancel },
        React__default["default"].createElement("form", { ref: formRef, onSubmit: async (e) => {
                e.preventDefault();
                await submit();
            }, noValidate: true },
            children,
            React__default["default"].createElement("input", { type: "submit", hidden: true }))));
}

const defaultOptions$1 = {
    title: 'Alert',
};
function AlertDialogPresentation({ show, confirmation, proceed, options, }) {
    const { title, modalClass } = lodash.defaults({ ...options }, defaultOptions$1);
    const closeRef = React.useRef(() => {
        /* no-op */
    });
    useEventListener__default["default"]('keypress', (e) => {
        if (e.key === 'Enter') {
            closeRef.current();
        }
    });
    if (!show)
        return null;
    return (React__default["default"].createElement(Dialog, { title: title, onClose: proceed, modalFooter: React__default["default"].createElement("button", { className: "btn btn-primary", onClick: () => closeRef.current(), type: "button" }, "OK"), closeRef: closeRef, focusFirst: true, focusFirstOptions: { additionalTagNames: ['button'] }, modalClass: modalClass }, confirmation));
}
// confirmable HOC pass props `show`, `dismiss`, `cancel` and `proceed` to your component
const ConfirmableDialog$1 = reactConfirm.confirmable(AlertDialogPresentation);
const _confirm = reactConfirm.createConfirmation(ConfirmableDialog$1);
/**
 * Imperative-style alert function backed by `react-confirm`.
 *
 * @returns a Promise which resolves when the alert is closed.
 */
function alert(content, options) {
    return _confirm({ options, confirmation: content }).then(() => undefined); // ignore return value
}

const defaultOptions = {
    title: 'Confirm',
};
function ConfirmDialogPresentation({ show, confirmation, loading = false, isStandalone = false, options, closeRef: propsCloseRef, ...otherProps }) {
    const privateCloseRef = React.useRef(() => { });
    const closeRef = propsCloseRef !== null && propsCloseRef !== void 0 ? propsCloseRef : privateCloseRef;
    const proceedCalledRef = React.useRef(false);
    let proceed;
    let onClose;
    if (isStandalone) {
        proceed = otherProps.proceed;
        onClose = () => {
            if (!proceedCalledRef.current)
                otherProps.cancel();
        };
    }
    else {
        proceed = () => {
            proceedCalledRef.current = true;
            closeRef.current();
        };
        onClose = () => {
            if (proceedCalledRef.current) {
                otherProps.proceed();
            }
            else {
                // important: we want to be able to await our confirm function, so call cancel
                // instead of dismiss so that closing the dialog results in the promise being rejected.
                // react-confirm does not resolve or reject if you call dismiss()
                otherProps.cancel();
            }
        };
    }
    const { cancelButtonText, actionButtonText, actionButtonClass, modalClass, title } = lodash.defaults({ ...options }, defaultOptions);
    if (!show)
        return null;
    return (React__default["default"].createElement(ActionDialog, { title: title, onClose: onClose, closeRef: closeRef, actionButtonText: actionButtonText, actionButtonClass: actionButtonClass, cancelButtonText: cancelButtonText, action: proceed, actionInProgress: loading, focusFirst: true, focusFirstOptions: { additionalTagNames: ['button'] }, modalClass: modalClass }, confirmation));
}
// confirmable HOC pass props `show`, `dismiss`, `cancel` and `proceed` to your component
const ConfirmableDialog = reactConfirm.confirmable(ConfirmDialogPresentation);
const confirm0 = reactConfirm.createConfirmation(ConfirmableDialog);
/** Imperative-style confirm dialog backed by `react-confirm`. */
function confirm(confirmation, options) {
    return confirm0({ options, confirmation }).then(() => undefined); // ignore return value
}
/** Standalone confirm dialog that does not use `react-confirm`. */
function ConfirmDialog({ confirmation, proceed, cancel, actionButtonText, actionButtonClass, loading = false, title, cancelButtonText, closeRef, modalClass, }) {
    const options = {
        title,
        actionButtonText,
        actionButtonClass,
        cancelButtonText,
        modalClass,
    };
    return (React__default["default"].createElement(ConfirmDialogPresentation, { isStandalone: true, confirmation: confirmation, proceed: proceed, cancel: cancel, dismiss: () => {
            throw new Error('ConfirmDialogPresentation called dismiss(). This should never happen!');
        }, show: true, loading: loading, options: options, closeRef: closeRef }));
}

/**
 * Makes a table cell into a link.
 *
 * `TdLink` has styles that are required for it to work correctly.
 *
 * ```
 * <table className="table table-hover table-td-link">
 *     <tbody>
 *         {products.map(p => {
 *             const Td = getTdLink('/product/detail/' + p.id)
 *
 *             return (
 *                 <tr key={p.id}>
 *                     <Td>{p.id}</Td>
 *                     <Td>{p.name}</Td>
 *                     <Td>{p.stock}</Td>
 *                 </tr>
 *             )
 *          })}
 *     </tbody>
 * </table>
 * ```
 */
function getTdLink(to, tdProps) {
    const TdLink = (props) => {
        const { children, className, ...otherProps } = props;
        const classes = ['td-link'];
        if (className)
            classes.push(className);
        return (React__default["default"].createElement("td", { ...tdProps },
            React__default["default"].createElement(reactRouterDom.Link, { to: to, className: classes.join(' '), ...otherProps }, children)));
    };
    return TdLink;
}

const delayMs = 200;
/**
 * The component that is displayed while a tab is loading in [[`TabManager`]].
 */
function TabContentLoading(props) {
    const [pastDelay, setPastDelay] = React.useState(false);
    React.useEffect(() => {
        const timer = window.setTimeout(() => {
            setPastDelay(true);
        }, delayMs);
        return () => {
            window.clearTimeout(timer);
        };
    }, []);
    // We're doing this weird thing with two LoadingIcons so that
    // - the height of the component doesn't change when pastDelay becomes true
    // - the real LoadingIcon's fadeIn animation doesn't play while it is invisible
    return (React__default["default"].createElement(ItiReactContext.Consumer, null, (data) => {
        var _a;
        // If no render prop was supplied, fallback to the context's renderLoadingIndicator
        const renderLoadingIndicator = (_a = props.renderLoadingIndicator) !== null && _a !== void 0 ? _a : data.renderLoadingIndicator;
        return (React__default["default"].createElement("div", { className: "tab-content-loading" },
            React__default["default"].createElement("div", { className: pastDelay ? 'd-none' : 'invisible' }, renderLoadingIndicator()),
            React__default["default"].createElement("div", { className: pastDelay ? '' : 'd-none' }, renderLoadingIndicator())));
    }));
}

function TabLink(props) {
    const { tab, current, onClick } = props;
    const anchorClasses = ['nav-link'];
    if (current === tab[0])
        anchorClasses.push('active');
    if (tab.length === 3) {
        const tabOptions = tab[2];
        if (tabOptions.className)
            anchorClasses.push(tabOptions.className);
    }
    return (React__default["default"].createElement("li", { className: "nav-item" },
        React__default["default"].createElement(LinkButton, { className: anchorClasses.join(' '), onClick: onClick }, tab[1])));
}
/**
 * A presentational component for building user interfaces with tabs.
 *
 * Usually you want to use [[`TabManager`]] which builds on top of `TabLayout`.
 */
function TabLayout({ tabs, children, tab, onTabClick, tabContentRef, tabContentStyle, className, }) {
    const navClasses = ['nav', 'nav-tabs'];
    const classes = ['tab-layout'];
    if (className)
        classes.push(className);
    return (React__default["default"].createElement("div", { className: classes.join(' ') },
        React__default["default"].createElement("ul", { className: navClasses.join(' ') }, tabs.map((t) => (React__default["default"].createElement(TabLink, { key: t[0], tab: t, current: tab, onClick: () => onTabClick(t[0]) })))),
        React__default["default"].createElement("div", { className: "tab-content", ref: tabContentRef, style: tabContentStyle }, children)));
}

const defaultUrlParamName = 'tab';
/** Determines the current tab from the `Location` object. */
function getTabFromLocation(tabs, location, options) {
    if (tabs.length === 0)
        throw new Error('tabs array cannot be empty.');
    const { defaultTabName, urlParamName } = lodash.defaults(options, {
        urlParamName: defaultUrlParamName,
    });
    const searchParams = new URLSearchParams(location.search);
    const tabParam = searchParams.get(urlParamName);
    if (tabParam && tabs.some((t) => t[0] === tabParam))
        return tabParam;
    if (defaultTabName)
        return defaultTabName;
    return tabs[0][0];
}
// When the user switches to a tab that needs to load first, keep the height
// of the tab-content the same until the new tab finishes loading to avoid
// jarring changes in height
function useSmoothTabTransition(renderTabs, tab) {
    const tabContentRef = React.useRef(null);
    const [explicitTabContentHeight, setExplicitTabContentHeight] = React.useState();
    const [setHeightTo, setSetHeightTo] = React.useState();
    // When a new tab is about to mount, get the height of the tabContent
    // BEFORE the tabs actually switch
    function newTabWillMount() {
        if (tabContentRef.current) {
            const height = tabContentRef.current.offsetHeight;
            if (typeof height === 'number')
                setSetHeightTo(height);
        }
    }
    // Explicitly set the tabContent height immediately after the new tab becomes
    // visible. useLayoutEffect is essential here because we want the effect to
    // run *before the browser paints*.
    React.useLayoutEffect(() => {
        if (typeof setHeightTo === 'number') {
            setExplicitTabContentHeight(setHeightTo);
            setSetHeightTo(undefined);
        }
    }, [setHeightTo]);
    const currentRenderTab = renderTabs.find((rt) => rt[0] === tab);
    // Set explicit height to undefined when new tab becomes ready
    React.useLayoutEffect(() => {
        if (typeof explicitTabContentHeight === 'number') {
            if (currentRenderTab && currentRenderTab[1]) {
                setExplicitTabContentHeight(undefined);
            }
        }
    }, [explicitTabContentHeight, currentRenderTab]);
    return { tabContentRef, explicitTabContentHeight, newTabWillMount };
}
/**
 * A high-level component for creating user interfaces with tabs.
 *
 * - It stores the current tab name as a URL search params.
 * - It doesn't mount the a tab until the user actually visits that tab. (You can disable this with by setting `mountAllTabs` to true.)
 * - It shows a loading indicator if a tab needs to load data before it can render content.
 */
function TabManager({ tabs, children, defaultTabName, renderLoadingIndicator, mountAllTabs = false, urlParamName = defaultUrlParamName, displaySingleTab = true, className, }) {
    const history = reactRouterDom.useHistory();
    const location = reactRouterDom.useLocation();
    let tab = '';
    if (tabs.length > 0) {
        tab = getTabFromLocation(tabs, location, { defaultTabName, urlParamName });
    }
    const [mountedTabs, setMountedTabs] = React.useState([tab]);
    const { tabContentRef, explicitTabContentHeight, newTabWillMount } = useSmoothTabTransition(children, tab);
    React.useEffect(() => {
        if (mountAllTabs)
            return;
        if (!mountedTabs.includes(tab)) {
            setMountedTabs((mountedTabs) => [...mountedTabs, tab]);
        }
    }, [mountedTabs, tab, mountAllTabs]);
    function onTabClick(tabName) {
        if (!mountedTabs.includes(tabName))
            newTabWillMount();
        const searchParams = new URLSearchParams(location.search);
        searchParams.set(urlParamName, tabName);
        history.replace({
            ...location,
            search: searchParams.toString(),
        });
    }
    function renderTab(theRenderTab) {
        const [thisTabName, ready, reactNode] = theRenderTab;
        if (!mountAllTabs && !mountedTabs.includes(thisTabName))
            return null;
        return (React__default["default"].createElement("div", { style: {
                display: tab === thisTabName ? undefined : 'none',
            }, className: !ready ? 'render-tab-loading' : undefined, key: thisTabName },
            !ready && (React__default["default"].createElement(TabContentLoading, { renderLoadingIndicator: renderLoadingIndicator })),
            React__default["default"].createElement("div", { className: ready ? '' : 'd-none' }, reactNode)));
    }
    if (tabs.length === 1 && !displaySingleTab) {
        if (!children || children.length === 0)
            return null;
        // Display contents without a tab or border
        return renderTab(children[0]);
    }
    return (React__default["default"].createElement(TabLayout, { tabs: tabs, tab: tab, onTabClick: onTabClick, tabContentRef: tabContentRef, tabContentStyle: typeof explicitTabContentHeight === 'number'
            ? { height: explicitTabContentHeight }
            : undefined, className: className }, children && children.map(renderTab)));
}

/** A component that displays an address. */
function AddressDisplay({ address, }) {
    if (!address)
        return null;
    return (React__default["default"].createElement("div", { className: "address-display" },
        address.line1 && React__default["default"].createElement("p", null, address.line1),
        address.line2 && React__default["default"].createElement("p", null, address.line2),
        React__default["default"].createElement("p", null, itiReactCore.formatAddressLine3(address)),
        itiReactCore.isCanadianPostalCode(address.postalCode) && React__default["default"].createElement("p", null, "Canada")));
}

/**
 * An easy to use checkbox component.
 *
 * ```
 * <FormCheck
 *     checked={myBoolean}
 *     onChange={() => setMyBoolean(b => !b)}
 *     label="My checkbox"
 * />
 * ```
 *
 * `inline` is true by default and should be set to false if you want to display
 * a vertical stack of checkboxes.
 */
const FormCheck = React__default["default"].memo(({ name, label, defaultChecked, checked, onChange, readOnly, className, autoComplete, tabIndex, enabled = true, inline = true, }) => {
    const idRef = React.useRef(itiReactCore.getGuid());
    const classes = ['form-check'];
    if (inline)
        classes.push('form-check-inline');
    if (className)
        classes.push(className);
    return (React__default["default"].createElement("div", { className: classes.join(' ') },
        React__default["default"].createElement("input", { type: "checkbox", className: "form-check-input", id: idRef.current, name: name, defaultChecked: defaultChecked, checked: checked, onChange: onChange, disabled: !enabled, readOnly: readOnly, autoComplete: autoComplete, tabIndex: tabIndex }),
        React__default["default"].createElement("label", { className: "form-check-label user-select-none", htmlFor: idRef.current }, label)));
});

/**
 * Shows a "✔ Saved" message next to your "Save changes" button to show that
 * user that the change went through. The message fades in and out.
 *
 * ```
 * const showSavedMessageRef = useRef(() => {})
 *
 * function saveChanges() {
 *     // call the API
 *
 *     showSavedMessageRef.current()
 * }
 *
 * return <div>
 *     ...
 *      <SavedMessage
 *          showSavedMessageRef={showSavedMessageRef}
 *          className="saved-message-me"
 *      />
 *     ...
 * </div>
 * ```
 */
function SavedMessage({ showSavedMessageRef, className, }) {
    const [show, setShow] = React.useState(false);
    const timerRef = React.useRef();
    function onSave() {
        setShow(true);
        clearTimeout(timerRef.current);
        timerRef.current = window.window.setTimeout(() => setShow(false), 1800);
    }
    React.useEffect(() => {
        showSavedMessageRef.current = onSave;
    });
    React.useEffect(() => () => {
        clearTimeout(timerRef.current);
    }, []);
    const classes = ['saved-message'];
    if (className)
        classes.push(className);
    return (React__default["default"].createElement("div", { className: classes.join(' '), style: { opacity: show ? 1 : 0 } },
        React__default["default"].createElement(reactFontawesome.FontAwesomeIcon, { icon: freeSolidSvgIcons.faCheck }),
        " Saved"));
}

/**
 * Renders an icon that you can click to copy some text.
 *
 * Relies on FontAwesome and `@tippyjs/react`.
 */
function ClickToCopy({ text, className }) {
    const [copied, setCopied] = React.useState(false);
    function copy() {
        copyToClipboard__default["default"](text);
        setCopied(true);
    }
    React.useEffect(() => {
        if (!copied)
            return undefined;
        const timer = window.setTimeout(() => setCopied(false), 2000);
        return () => {
            window.clearTimeout(timer);
        };
    }, [copied]);
    const tooltip = copied ? 'Copied!' : 'Click to copy';
    const classes = ['click-to-copy'];
    if (className)
        classes.push(className);
    return (React__default["default"].createElement(Tippy__default["default"], { content: tooltip, hideOnClick: false },
        React__default["default"].createElement("div", { className: classes.join(' '), onClick: copy, onKeyDown: (e) => {
                if (e.key === 'Enter')
                    copy();
            }, role: "button", tabIndex: 0 },
            React__default["default"].createElement(reactFontawesome.FontAwesomeIcon, { icon: freeSolidSvgIcons.faCopy }))));
}

const ALL = 'ALL';
const allPageSize = 10000;
function getPageSizeOptions(pageSizes, showAllOption) {
    const options = lodash.sortBy(pageSizes, (x) => x).map((x) => ({
        value: x,
        label: x.toString(),
    }));
    if (showAllOption)
        options.push({ value: ALL, label: 'All' });
    return options;
}
function getSkipTake(page, pageSize) {
    return {
        skip: (page - 1) * pageSize,
        take: pageSize,
    };
}
// TODO convert to Redux Toolkit or completely vanilla (no action helper library)
/** @internal */
const pageActions = {
    setPage: typesafeActions.createAction('SET_PAGE')(),
    setPageSize: typesafeActions.createAction('SET_PAGE_SIZE')(),
    showAllItems: typesafeActions.createAction('SHOW_ALL_ITEMS')(),
};
/** @internal */
const pageReducer = typesafeActions.createReducer({ page: 0, pageSize: 0 } // never used
)
    .handleAction(pageActions.setPage, (state, action) => ({
    ...state,
    page: action.payload,
}))
    .handleAction(pageActions.setPageSize, (state, action) => {
    const firstVisibleItemIndex = getSkipTake(state.page, state.pageSize).skip;
    const pageSize = action.payload;
    return {
        page: Math.floor(firstVisibleItemIndex / pageSize) + 1,
        pageSize,
    };
})
    .handleAction(pageActions.showAllItems, () => ({
    page: 1,
    pageSize: allPageSize, // for safety, limit the number of items that can be displayed
}));
/**
 * A pagination control that lets the user select how many items to show on a
 * page.
 *
 * The default page sizes are configured in [[`ItiReactContext`]] and can be
 * overridden with the `pageSizes` prop.
 *
 * You can show an "All" option by setting `showAllOption` to true. "All"
 * actually means a `pageSize` of 10000.
 */
function ConfigurablePager({ page, pageSize, onChange, totalPages, enabled, showAllOption = false, ...props }) {
    var _a;
    const itiReactContext = React.useContext(ItiReactContext);
    const pageSizes = (_a = props.pageSizes) !== null && _a !== void 0 ? _a : itiReactContext.configurablePager.pageSizes;
    const selectIdRef = React.useRef(itiReactCore.getGuid());
    function dispatch(action) {
        const updated = pageReducer({ page, pageSize }, action);
        onChange(updated.page, updated.pageSize);
    }
    return (React__default["default"].createElement("div", { className: "pagination-container" },
        React__default["default"].createElement("div", { className: "configurable-pager" },
            React__default["default"].createElement("label", { htmlFor: selectIdRef.current }, "Items per page"),
            React__default["default"].createElement(ValidatedSelect, { id: selectIdRef.current, name: "pageSize", value: pageSize === allPageSize ? ALL : pageSize, onChange: (pageSize) => {
                    if (pageSize === ALL) {
                        dispatch(pageActions.showAllItems());
                    }
                    else {
                        dispatch(pageActions.setPageSize(pageSize));
                    }
                }, options: getPageSizeOptions(pageSizes, showAllOption), showValidation: false, validators: [], enabled: enabled }),
            React__default["default"].createElement(Pager, { page: page, onPageChange: (page) => dispatch(pageActions.setPage(page)), totalPages: totalPages, enabled: enabled, containerClassName: "" }))));
}

function BreadcrumbItemComponent({ item, last, }) {
    return (React__default["default"].createElement("li", { className: `breadcrumb-item ${last ? '' : 'active'}` }, last ? React__default["default"].createElement("span", null, item.label) : React__default["default"].createElement(reactRouterDom.Link, { to: item.path }, item.label)));
}
/** A navigation breadcrumbs component. */
function Breadcrumbs({ items }) {
    if (!items || items.length <= 1) {
        return React__default["default"].createElement("div", { className: "breadcrumbs" });
    }
    return (React__default["default"].createElement("div", { className: "breadcrumbs" },
        React__default["default"].createElement("ol", { className: "breadcrumb" }, items.map((item, i) => (React__default["default"].createElement(BreadcrumbItemComponent, { item: item, last: i === items.length - 1, key: item.path }))))));
}

/**
 * Displays a dollar sign at the beginning of a text input.
 */
function MoneyInputContainer({ children, }) {
    return (React__default["default"].createElement("div", { className: "money-input-container" },
        React__default["default"].createElement("div", { className: "dollar-sign" }, "$"),
        children));
}

/**
 * Displays an input along with a unit.
 *
 * ```
 * <UnitInputContainer unit="km">
 *     <ValidatedInput />
 * </UnitInputContainer>
 * ```
 */
function UnitInputContainer({ children, unit, className, }) {
    const classes = ['unit-input-container'];
    if (className)
        classes.push(className);
    return (React__default["default"].createElement("div", { className: classes.join(' ') },
        children,
        React__default["default"].createElement("div", { className: "unit" }, unit)));
}

/**
 * Renders a form input and corresponding label.
 *
 * The children of this component can either be a JSX element or a function that
 * maps `id` string to a JSX element (better for accessibility).
 */
function FormGroup({ label, className, style, loading, optional = false, ...props }) {
    const idRef = React.useRef(itiReactCore.getGuid());
    const { renderLoadingIndicator } = React.useContext(ItiReactContext);
    let labelInner = label;
    if (optional) {
        labelInner = (React__default["default"].createElement("span", null,
            label,
            " ",
            React__default["default"].createElement("span", { className: "optional" }, "(optional)")));
    }
    let children;
    if (typeof props.children === 'function') {
        const renderProp = props.children;
        children = renderProp(idRef.current);
    }
    else {
        children = props.children;
    }
    return (React__default["default"].createElement("div", { className: `form-group ${className || ''}`, style: style, "data-testid": props['data-testid'] },
        label && (React__default["default"].createElement("label", { className: "form-label", htmlFor: idRef.current },
            labelInner,
            ' ',
            typeof loading !== 'undefined' &&
                loading &&
                renderLoadingIndicator())),
        children));
}

/** Tests if `Location` object are equivalent. */
function areLocationsEqualIgnoringKey(a, b) {
    return (a.pathname === b.pathname &&
        a.search === b.search &&
        a.hash === b.hash &&
        lodash.isEqual(a.state, b.state));
}
function stripTrailingSlash(path) {
    return path.endsWith('/') ? path.slice(0, -1) : path;
}
/**
 * Tests if two paths are equivalent by removing trailing slashes and converting to
 * all lowercase.
 */
function arePathsEqual(path1, path2) {
    function normalize(p) {
        return stripTrailingSlash(p).toLowerCase();
    }
    return normalize(path1) === normalize(path2);
}

/**
 * A component factory that returns an `AsyncRouter`. When using `AsyncRouter` and
 * a link changes the route, the new page is mounted but not displayed until it has
 * finished loading data and calls `onReady`. A progress bar at the top of the page
 * (usually NProgress) indicates to the user that the new page is loading.
 *
 * It is fine if a page calls `onReady` multiple times.
 *
 * Example:
 * ```
 * export interface OnReadyArgs {
 *     title: string
 * }
 *
 * const AsyncRouter = getAsyncRouter<OnReadyArgs>()

 * export function MyAsyncRouter(): React.ReactElement {
 *     const dispatch = useDispatch()
 *
 *     const onReady = useCallback(
 *         ({ title }: OnReadyArgs): void => {
 *             const _window = window as unknown as WindowWithGlobals
 *             if (_window.loadingScreen) {
 *                 _window.loadingScreen.finish(false, _window.onLoadingScreenHidden)
 *                 _window.loadingScreen = undefined
 *             }
 *
 *             updateTitle(title)
 *         },
 *         []
 *     )
 *
 *     return (
 *         <AsyncRouter
 *             renderRoutes={(args) => <Routes {...args} />}
 *             renderLayout={(children) => (
 *                 <Layout>
 *                     {children}
 *                 </Layout>
 *             )}
 *             getLocationKey={getLocationKey}
 *             onNavigationStart={() => NProgress.start()}
 *             onNavigationDone={() => NProgress.done()}
 *             onReady={onReady}
 *         />
 *     )
 * }
 *
 * function getLocationKey(location: Location): string {
 *     const locationMatchesPath = (p: string): boolean =>
 *         !!matchPath(location.pathname, {
 *             path: p,
 *             exact: true,
 *         })
 *
 *     if (locationMatchesPath(jobPaths.jobBoard)) return '/job/board'
 *
 *     return location.pathname.toLowerCase()
 * }
 * ```
 *
 * ### `getLocationKey`
 *
 * - If you want to be able to change the route parameters without the page
 *   unmounting and remounting, you should implement `getLocationKey`,
 *   so that the page has the same location key regardless of the route
 *   params.
 *
 *   Example: you want to be able to navigate from `/job/0` to `/job/1`
 *   without the page remounting. You should make `getLocationKey` return
 *   `'/job'` for any path that matches `'/job/:page?`. Use `matchPath`
 *   for this.
 *
 *   LocationKey is NOT the same as the `key` property of the `location` object.
 *
 * - When implementing `getLocationKey`, if the current path does not correspond to
 *   a route in your application, you must return the current path without any modifications.
 *
 *   Consider this example to see why you must return the path unmodified. Say you have a route
 *   `/job/:page?` and you have implemented `getLocationKey` to return `'/job'` for any
 *   path that starts with `/job`. The user enters `/job/bogus/path/here` into the URL bar.
 *   The "Page not found" page will be rendered. Then the user clicks the navbar link to the job list.
 *   The location key does not change when the location changes, so `AsyncRouter` does not unmount the
 *   previous page and mount the new page. But the two pages are actually different! So yeah
 *   weird stuff can happen if you implement `getLocationKey` like this. Always use `matchPath` in
 *   `getLocationKey`!
 *
 * ### Test Cases
 *
 * 1. Normal navigation.
 * 2. Double click a link - navigation should still occur like normal.
 * 3. Click a link and press browser back button while page is loading.
 * 4. Click a link and then click a different link while page is loading.
 * 5. Go to the "Spam onReady" page in `test-website` and follow the steps.
 * 6. Redirect to self: click the ITI logo in `test-website` while on `/home/index`.
 *    The logo is a link to `/`. When the path is `/`, a redirect to `/home/index` is
 *    rendered, putting you back where you started.
 *
 * @typeParam TOnReadyArgs the arguments your pages will pass to `onReady`
 */
function getAsyncRouter() {
    return function AsyncRouter(props) {
        const { renderRoutes, renderLayout, getLocationKey } = props;
        const location = reactRouterDom.useLocation();
        const [displayedLocation, setDisplayedLocation] = React.useState(location);
        const [loadingLocation, setLoadingLocation] = React.useState();
        const [initialLocationCalledOnReady, setInitialLocationCalledOnReady] = React.useState(false);
        // default to true since initial page is loading
        const [navigationInProgress, setNavigationInProgress] = React.useState(true);
        React.useEffect(() => {
            if (typeof displayedLocation !== 'undefined') {
                const locationChanged = !areLocationsEqualIgnoringKey(displayedLocation, location);
                const locationKeyChanged = getLocationKey(displayedLocation) !== getLocationKey(location);
                if (locationChanged) {
                    if (locationKeyChanged) {
                        setLoadingLocation(location);
                    }
                    else {
                        // Should not reload page when location key is the same - this is the whole
                        // point of location keys
                        setDisplayedLocation(location);
                    }
                }
            }
        }, [displayedLocation, location, getLocationKey]);
        function onNavigationStart() {
            setNavigationInProgress(true);
            props.onNavigationStart();
        }
        function onNavigationDone() {
            setNavigationInProgress(false);
            setLoadingLocation(undefined);
            window.scrollTo(0, 0);
            props.onNavigationDone();
        }
        const prevLocation = itiReactCore.usePrevious(location);
        React.useEffect(() => {
            if (!navigationInProgress &&
                prevLocation &&
                getLocationKey(location) !== getLocationKey(prevLocation)) {
                // normal navigation start
                onNavigationStart();
            }
        });
        React.useEffect(() => {
            if (loadingLocation &&
                getLocationKey(location) === getLocationKey(displayedLocation)) {
                // We got redirected to the page we're already on
                onNavigationDone();
            }
        });
        function onReady(location, args) {
            const isForLoadingLocation = loadingLocation && areLocationsEqualIgnoringKey(location, loadingLocation);
            // ignore any unexpected calls to onReady.
            // if the user begins navigation to one page, but then interrupts the navigation by clicking
            // on a link, we can still get an onReady call from the first page. This call must be ignored,
            // or else weirdness will occur.
            //
            // this can also happen when a page calls onReady multiple times, for example, the page calls
            // onReady every time a query completes. Calling onReady multiple times is harmless, and as such,
            // no warning should be displayed.
            if (isForLoadingLocation || !initialLocationCalledOnReady) {
                // normal navigation done
                setDisplayedLocation(location);
                setInitialLocationCalledOnReady(true);
                onNavigationDone();
                // Necessary to support dialogs that have links
                cleanupImproperlyClosedDialog();
                props.onReady(args);
            }
        }
        const pages = [
            renderRoutes({
                location: displayedLocation,
                key: getLocationKey(displayedLocation),
                ready: initialLocationCalledOnReady,
                onReady: (args) => onReady(displayedLocation, args),
            }),
        ];
        if (loadingLocation &&
            getLocationKey(loadingLocation) !== getLocationKey(displayedLocation)) {
            pages.push(renderRoutes({
                location: loadingLocation,
                key: getLocationKey(loadingLocation),
                ready: false,
                onReady: (args) => onReady(loadingLocation, args),
            }));
        }
        return renderLayout(pages);
    };
}

/**
 * A React component that adds a URL search param (usually `error`) if the `error`
 * prop is non-null. If the `error` prop is null, the URL search param is removed.
 *
 * Usually, the `error` prop will come from the Redux store. This allows you to display
 * an error page whenever an error occurs anywhere in your application.
 *
 * ### Test Cases
 *
 * 1. When an error occurs, the error param is added to the URL and the error page is
 *    displayed.
 * 2. When the URL contains the error param and user refreshes the page, the error
 *    param is removed and the error page is not displayed.
 * 3. When the user is on the error page and clicks back, they are returned to the page
 *    they were on when the error occurred.
 * 4. This tests the errorKey functionality.
 *    a. Error occurs, and error page is displayed.
 *    b. User navigates to some other page.
 *    c. Another error occurs. Make sure the error page is displayed.
 */
function ErrorRouteSynchronizer({ errorUrlParamName, error, }) {
    const history = reactRouterDom.useHistory();
    const location = reactRouterDom.useLocation();
    const prevError = itiReactCore.usePrevious(error);
    React.useEffect(() => {
        const urlSearchParams = new URLSearchParams(location.search);
        const errorUrlParamExists = urlSearchParams.has(errorUrlParamName);
        if (error) {
            if (error !== prevError && !errorUrlParamExists) {
                // Add error URL param if an error was just set in the Redux state
                urlSearchParams.append(errorUrlParamName, '');
                // Push so that clicking back takes you to the page you were on
                history.push(`${location.pathname}?${urlSearchParams.toString()}`);
            }
        }
        else if (errorUrlParamExists) {
            // Remove error URL param if no error in Redux state
            urlSearchParams.delete(errorUrlParamName);
            history.replace(`${location.pathname}?${urlSearchParams.toString()}`);
        }
    }, [error, prevError, history, location, errorUrlParamName]);
    return null;
}

/**
 * Returns true if all values in the object are true. For use with
 * [[`useReadiness`]].
 */
function allReady(readiness) {
    return Object.values(readiness).every((v) => v);
}
/**
 * Used to call `onReady` when all of the components/queries in a page are ready.
 *
 * ```
 * const [onChildReady, readiness] = useReadiness(
 *     {
 *         a: false, b: false, c: false
 *     },
 *     readiness => {
 *         if (allReady(readiness)) {
 *             onReady({ ... })
 *         }
 *     },
 * )
 * ```
 */
function useReadiness(defaultValue, onChange) {
    const [readiness, setReadiness] = React.useState(defaultValue);
    const onChangeRef = React.useRef(onChange);
    React.useEffect(() => {
        onChangeRef.current = onChange;
    });
    React.useEffect(() => {
        onChangeRef.current(readiness);
    }, [readiness]);
    const onChildReady = React.useCallback((delta) => {
        setReadiness((readiness) => lodash.merge({ ...readiness }, delta));
    }, [setReadiness]);
    return [onChildReady, readiness];
}

/**
 * A hook that combines three things that need to be implemented when using pagination:
 *
 * - Resets page when filters change
 * - Returns total number of pages
 * - If the current page has no items and is not the first page, decrements the page
 *   (prevent non-existent pages). This situation occurs when someone deletes items
 *   while the user is viewing a paginated list.
 *
 * ```
 * const totalPages = usePaginationHelpers(...)
 * ```
 */
function usePaginationHelpers(options) {
    const { queryParams, onPageChange, firstPage, items, totalCount, pageSizeWhenItemsRetrieved, } = lodash.defaults(options, {
        firstPage: 1,
    });
    const prevQueryParamsRef = React.useRef();
    React.useEffect(() => {
        if (prevQueryParamsRef.current) {
            const newPage = itiReactCore.resetPageIfFiltersChanged(prevQueryParamsRef.current, queryParams, firstPage).page;
            if (queryParams.page !== newPage)
                onPageChange(newPage);
        }
        prevQueryParamsRef.current = queryParams;
    }, [queryParams, firstPage, onPageChange]);
    const pageHasItems = items.length !== 0;
    React.useEffect(() => {
        itiReactCore.preventNonExistentPage({
            page: queryParams.page,
            pageHasItems,
            onPageChange,
            firstPage,
        });
    }, [queryParams.page, pageHasItems, onPageChange, firstPage]);
    return itiReactCore.getTotalPages(totalCount, pageSizeWhenItemsRetrieved);
}

/** @internal */
function hasAncestor(target, selector) {
    const parent = target.parentElement;
    if (!parent)
        return false;
    if (parent.matches(selector))
        return true;
    return hasAncestor(parent, selector);
}
/**
 * A hook that detects when the user has clicked outside of a popover,
 * indicating that the popover should be closed.
 */
function usePopoverClickListener({ visible, onOutsideClick, popoverSelector = '.iti-react-popover', }) {
    function onClick(e) {
        if (!visible)
            return;
        if (!e.target)
            return;
        const target = e.target;
        // Hack to prevent clicking a react select option from closing the popover
        if (hasAncestor(target, '[class$="-menu"]'))
            return;
        const inPopover = target.matches(popoverSelector) || hasAncestor(target, popoverSelector);
        if (!inPopover) {
            // To understand why we use defer here, consider what would happen if we
            // did NOT user defer. When the reference element is clicked, onOutsideClick
            // would be called immediately and the popover would be hidden. Then the
            // onClick handler of the reference element would fire, immediately showing
            // the popover again.
            // We used to prevent this using e.stopPropagation(), but stopping
            // propagation causes LinkButton act like a link to # instead of
            // a button.
            lodash.defer(onOutsideClick);
        }
    }
    useEventListener__default["default"]('click', onClick);
}

exports.ActionDialog = ActionDialog;
exports.AddressDisplay = AddressDisplay;
exports.AddressInput = AddressInput;
exports.AddressValidators = AddressValidators;
exports.AsyncSelectUtil = AsyncSelectUtil;
exports.AsyncSelectValidators = AsyncSelectValidators;
exports.BooleanRadioInput = BooleanRadioInput;
exports.BooleanRadioValidators = BooleanRadioValidators;
exports.Breadcrumbs = Breadcrumbs;
exports.ClickToCopy = ClickToCopy;
exports.ConfigurablePager = ConfigurablePager;
exports.ConfirmDialog = ConfirmDialog;
exports.DateInput = DateInput;
exports.DateInputNoPicker = DateInputNoPicker;
exports.DateInputNoPickerValidators = DateInputNoPickerValidators;
exports.DateValidators = DateValidators;
exports.DefaultClearButtonComponent = DefaultClearButtonComponent;
exports.Dialog = Dialog;
exports.EasyFormDialog = EasyFormDialog;
exports.ErrorRouteSynchronizer = ErrorRouteSynchronizer;
exports.FileInput = FileInput;
exports.FileValidators = FileValidators;
exports.FormCheck = FormCheck;
exports.FormGroup = FormGroup;
exports.InternalAddressValidators = InternalAddressValidators;
exports.ItiReactContext = ItiReactContext;
exports.LinkButton = LinkButton;
exports.MoneyInputContainer = MoneyInputContainer;
exports.MultiSelectValidators = MultiSelectValidators;
exports.Pager = Pager;
exports.PersonNameInput = PersonNameInput;
exports.PersonNameValidators = PersonNameValidators;
exports.PhoneInput = PhoneInput;
exports.RadioInput = RadioInput;
exports.RadioValidators = RadioValidators;
exports.SavedMessage = SavedMessage;
exports.SelectValidators = SelectValidators;
exports.SubmitButton = SubmitButton;
exports.TabContentLoading = TabContentLoading;
exports.TabLayout = TabLayout;
exports.TabManager = TabManager;
exports.TimeInput = TimeInput;
exports.TimeValidators = TimeValidators;
exports.TimeZoneInput = TimeZoneInput;
exports.TimeZoneValidators = TimeZoneValidators;
exports.UnitInputContainer = UnitInputContainer;
exports.ValidatedAsyncSelect = ValidatedAsyncSelect;
exports.ValidatedInput = ValidatedInput;
exports.ValidatedMultiSelect = ValidatedMultiSelect;
exports.ValidatedSelect = ValidatedSelect;
exports.ValidationFeedback = ValidationFeedback;
exports.alert = alert;
exports.allReady = allReady;
exports.areLocationsEqualIgnoringKey = areLocationsEqualIgnoringKey;
exports.arePathsEqual = arePathsEqual;
exports.canadianProvinces = canadianProvinces;
exports.cleanupImproperlyClosedDialog = cleanupImproperlyClosedDialog;
exports.confirm = confirm;
exports.dateInputFormat = dateInputFormat;
exports.dateTimeInputFormat = dateTimeInputFormat;
exports.defaultAddressInputValue = defaultAddressInputValue;
exports.defaultItiReactContextData = defaultItiReactContextData;
exports.defaultPersonNameInputValue = defaultPersonNameInputValue;
exports.defaultPostalCodeValidationOptions = defaultPostalCodeValidationOptions;
exports.defaultTimeInputValue = defaultTimeInputValue;
exports.filterOption = filterOption;
exports.formatDateInputNoPickerValue = formatDateInputNoPickerValue;
exports.getAsyncRouter = getAsyncRouter;
exports.getNonGroupOptions = getNonGroupOptions;
exports.getSelectStyles = getSelectStyles;
exports.getStateOptions = getStateOptions;
exports.getStates = getStates;
exports.getTabFromLocation = getTabFromLocation;
exports.getTdLink = getTdLink;
exports.getValidationClass = getValidationClass;
exports.hasAncestor = hasAncestor;
exports.isPostalCodeValid = isPostalCodeValid;
exports.pageActions = pageActions;
exports.pageReducer = pageReducer;
exports.parseDateInputNoPickerValue = parseDateInputNoPickerValue;
exports.phoneInputValidator = phoneInputValidator;
exports.postalCodeValidator = postalCodeValidator;
exports.stripTrailingSlash = stripTrailingSlash;
exports.timeInputValueFromDecimalHours = timeInputValueFromDecimalHours;
exports.timeInputValueToDecimalHours = timeInputValueToDecimalHours;
exports.useDebouncedAsyncValidationPending = useDebouncedAsyncValidationPending;
exports.usePaginationHelpers = usePaginationHelpers;
exports.usePopoverClickListener = usePopoverClickListener;
exports.useReadiness = useReadiness;
Object.keys(itiReactCore).forEach(function (k) {
    if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
        enumerable: true,
        get: function () { return itiReactCore[k]; }
    });
});
